/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/auth.c,v 1.6 1996/03/02 00:38:50 mcooper Exp $";
#endif	/* lint */

#include "defs.h"
#if 	defined(sunos5)
#include <rpc/svc_soc.h>
#endif	/* sunos5 */

/*
 * See if Name is an allowed network.
 * Name can be a network address or network name.
 */
static int
AUTHchkNet(HostEnt, Name)
    struct hostent	       *HostEnt;
    char		       *Name;
{
    struct netent	       *NetEnt;
    u_long			NetNum;
    char		      **Addrs;
    static struct in_addr	in_addr;

    NetNum = inet_network(Name);
    if (NetNum == -1) {
	/* Not a network address */
	NetEnt = getnetbyname(Name);
	if (!NetEnt) {
	    Error("No such network as `%s'.", Name);
	    return(-1);
	}
	NetNum = NetEnt->n_net;
    }

    if (HostEnt->h_addrtype == AF_INET) {
#if	defined(h_addr)
	for (Addrs = HostEnt->h_addr_list; Addrs && *Addrs; ++Addrs) {
	    (void) memcpy((char *)&in_addr, *Addrs, sizeof(in_addr));
	    if (NetNum == (u_long) inet_netof(in_addr))
		return(0);
	}
#else	/* Older systems */
	(void) memcpy((char *)&in_addr, (char *)HostEnt->h_addr, 
		      sizeof(in_addr));
	if (NetNum == (u_long) inet_netof(in_addr))
	    return(0);
#endif	/* h_addr */
    }

    /* No match found */
    return(1);
}

/*
 * Check to see if Name is an allowed host.
 * Name can be either an address or hostname.
 */
static int
AUTHchkHost(HostEnt, Name)
    struct hostent	       *HostEnt;
    char		       *Name;
{
    struct hostent	       *hp;
    static char			Host[BUFSIZ];
    char		      **Addrs;
    static struct in_addr	in_addr;

    if (HostEnt->h_addrtype == AF_INET) {
#if	defined(h_addr)
	for (Addrs = HostEnt->h_addr_list; Addrs && *Addrs; ++Addrs) {
	    memcpy(&in_addr, *Addrs, sizeof(in_addr));
	    if (EQ(Name, inet_ntoa(in_addr)))
		return(0);
	}
#else	/* We only have h_addr */
	memcpy(&in_addr, HostEnt->h_addr, HostEnt->h_length);
	if (EQ(Name, inet_ntoa(in_addr)))
	    return(0);
#endif	/* h_addr */
    }

    (void) strcpy(Host, HostEnt->h_name);
    hp = gethostbyname(Name);
    if (!hp) {
	if (Debug) Error("Unknown host/address: `%s'.", Name);
	return(-1);
    }

    if (EQ(Host, hp->h_name) || EQ(Name, hp->h_name))
	return(0);
    else
	/* No match */
	return(1);
}

/*
 * Check to see if host is in the access list.
 */
static int
AUTHhostOkay(HostEnt, AccessInfo)
    struct hostent	       *HostEnt;
    AccessInfo_t	       *AccessInfo;
{
    static char		        ErrBuf[BUFSIZ];
    KeyVal_t		       *Ptr;
    int				Okay = FALSE;

    if (!AccessInfo->List) {
	Error(
      "No access list in config file for `%s' - ALL accesses will be denied.",
	      AccessInfo->Name);
	return(-1);
    }

    for (Ptr = AccessInfo->List; !Okay && Ptr; Ptr = Ptr->Next) {
	if (EQC(Ptr->Key, KW_NET)) {
	    if (AUTHchkNet(HostEnt, Ptr->Value) == 0)
		Okay = TRUE;
	} else if (EQC(Ptr->Key, KW_HOST)) {
	    if (AUTHchkHost(HostEnt, Ptr->Value) == 0)
		Okay = TRUE;
	} else
	    Error("Unknown keyword in Access List: `%s'.", Ptr->Key);
    }

    return( (Okay) ? 0 : -1 );
}

/*
 * Get Client's Auth Info
 */
extern ClientInfo_t *
AUTHcheck(Request, AccessInfo)
    struct svc_req	       *Request;
    AccessInfo_t	       *AccessInfo;
{
    static char			HostName[BUFSIZ];
    struct sockaddr_in	       *ClientAddr;
    struct hostent	       *HostEnt;
    struct authunix_parms      *AuthUnix = NULL;
    ClientInfo_t	       *ClientInfo;
    SVCXPRT		       *Transport;
    char		       *ClientName = "(unknown)";

    if (!Request) {
	Error("No request information provided (Probably a broadcast).");
	return((ClientInfo_t *) NULL);
    }

    Transport = Request->rq_xprt;
    if (!Transport) {
	Error("No transport information available.");
	return((ClientInfo_t *) NULL);
    }
	
    /*
     * Get client's hostname passed by RPC (yuck!)
     */
    if (Request && Request->rq_clntcred)
	AuthUnix = (struct authunix_parms *) Request->rq_clntcred;
    if (AuthUnix && AuthUnix->aup_machname && AuthUnix->aup_machname[0])
	ClientName = AuthUnix->aup_machname;

    /*
     * XXX We should handle other AUTH_* types
     */
    if (Request->rq_cred.oa_flavor != AUTH_UNIX) {
	Error("%s: Non Unix authentication (%d)",
	      ClientName, Request->rq_cred.oa_flavor);
	svcerr_weakauth(Transport);
	return((ClientInfo_t *) NULL);
    }

    ClientAddr = svc_getcaller(Transport);
    if ((ntohs(ClientAddr->sin_port) >= IPPORT_RESERVED)) {
	Error("%s: Non reserved port request from port %d",
	      ClientName, ntohs(ClientAddr->sin_port));
	svcerr_weakauth(Transport);
	return((ClientInfo_t *) NULL);
    }

    HostEnt = gethostbyaddr( (char *)&ClientAddr->sin_addr, 
			    sizeof(ClientAddr->sin_addr), AF_INET);
    if (!HostEnt) {
	Error("Cannot lookup host entry for address %s [Claimed %s].",
	      inet_ntoa(ClientAddr->sin_addr), ClientName);
	svcerr_weakauth(Transport);
	return((ClientInfo_t *) NULL);
    }
    (void) strcpy(HostName, HostEnt->h_name);

    if (AUTHhostOkay(HostEnt, AccessInfo) != 0) {
	Error("%s access denied to %s", AccessInfo->Name, HostName);
	svcerr_weakauth(Transport);
	return((ClientInfo_t *) NULL);
    }

    ClientInfo = CLNTcreate();
    ClientInfo->HostName = strdup(HostName);
    ClientInfo->ClientName = strdup(ClientName);
    ClientInfo->AddrString = strdup( inet_ntoa(ClientAddr->sin_addr) );

    return(ClientInfo);
}
