/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/change.c,v 1.19 1996/03/02 00:39:01 mcooper Exp $";
#endif	/* lint */

/*
 * Password Management System client change functions
 */

#include "defs.h"

/*
 * Change something for UserName.
 */
int
Change(UserName, ReqUser, FacType, PCtype, ArgStr)
    char		       *UserName;
    char		       *ReqUser;
    FACtype_t			FacType;
    PCtype_t			PCtype;
    char		       *ArgStr;
{
    static SPMquery_t		Query;
    static SPMresult_t		Result;
    static char			CurPass[BUFSIZ];
    static char			ErrBuff[BUFSIZ];
    register int		Count;
    ClientInfo_t	       *Client;
    char		       *Master;
    char		       *Domain;
    char		       *Arg;
    char		       *What;
    int				Status = -1;
    Facility_t		       *Facility;
    char		     *(*Ask)();
    u_long			SpmProc;
    VERIFYstat_t		VerStat;

    Facility = FACget(UserName, FacType);
    if (!Facility)
	return(-1);

    Domain = GetDomain(Facility);
    if (!Domain)
	return(-1);

    if (SPMserver)
	Master = SPMserver;
    else if (Facility->GetServer)
	Master = (*Facility->GetServer)(Domain);

    switch (PCtype) {
    case PC_PWD:
	Ask = ASKnewpwd;
	SpmProc = SPMPROC_SETPWD;
	break;
    case PC_GECOS:	
	Ask = ASKgecos;
	SpmProc = SPMPROC_SETGECOS;
	break;
    case PC_SHELL:
	Ask = ASKshell;
	SpmProc = SPMPROC_SETSHELL;
	break;
    default:
	Error("Unknown PC type: %d.", PCtype);
	return(-1);
    }

    What = GetPCname(PCtype);
    Message("Changing %s %s for %s%s%s . . .", 
	    Facility->Name, What,
	    UserName, 
	    (Master) ? " on " : "", 
	    (Master) ? Master : "");

    /*
     * Try to start connection to server now to make sure it's up
     * before we ask user for information.
     */
    Client = CLNTconnect(Master);
    if (!Client)
	return(-1);

    /*
     * Ask user for current password
     */
    if (ASKpassword("Current Password:", CurPass, sizeof(CurPass)) < 0)
	return(-1);

    /*
     * Form query
     */
    QueryInit(&Query);
    Query.UserName = UserName;
    Query.ReqUser = ReqUser;
    Query.CurPass = CurPass;
    Query.Domain = Domain;
    Query.Facility = Facility->Name;

    /*
     * Query to see if current password is okay and if server
     * will talk to us.
     */
    if (CLNTspmCall(Client, SPMPROC_CHECKPWD, &Query, &Result, 0) < 0) {
	Error("Check password RPC call to `%s' failed.", Client->HostName);
	return(-1);
    }
    if (Result.Status != RS_OK) {
	Error("Server error on %s: %s.", Master, ResultErrStr(&Result));
	return(-1);
    }

    if (PCtype == PC_PWD) {
	/*
	 * Verify information with user.
	 * If VerStat == VS_OKAY or VerStat == VS_NOINFO, then we don't
	 * return an error.
	 */
	VerStat = VERIFYuser(&Query);
	if (VerStat == VS_ERROR || VerStat == VS_BADINFO) {
	    Error("Verification proceedure failed.");
	    exit(2);
	}
    }

    /*
     * Call appropriate function to ask user for change info.
     */
    for (Count = 0, Arg = NULL, ErrBuff[0] = CNULL; Count < MaxAsk && !Arg; 
	 ++Count) {
	Arg = (*Ask)(&Query, Facility, ArgStr, ErrBuff);
	if (ErrBuff[0]) {
	    Error("%s", ErrBuff);
	    return(-1);
	}
    }

    if (!Arg)
	return(-1);
    Query.Arg = Arg;

    /*
     * Send query 
     */
    if (CLNTspmCall(Client, SpmProc, &Query, &Result, SPM_RETRY_INTR) < 0) {
	Error("Set %s RPC call to `%s' failed.", What, Client->HostName);
	return(-1);
    }

    if (Result.Status != RS_OK) {
	Error("Server error on %s: %s.", Master, ResultErrStr(&Result));
	Status = -1;
    } else {
	if (Result.Message[0])
	    Message("%s", Result.Message);
	else
	    Message("Your %s %s has been changed.", Facility->Name, What);
	if (Facility->Type == FAC_NIS)
	    Message("This change may take up to two hours to take effect.");
	Status = 0;
    }

    return(Status);
}
