/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/clnt.c,v 1.15 1996/03/02 00:39:11 mcooper Exp $";
#endif	/* lint */

/*
 * Password Management System Client Functions
 */

#include "defs.h"

/*
 * Do the actual setup of connection to RPC server.
 */
ClientInfo_t *
_CLNTconnect(ServerName, RetryAmt)
    char		       *ServerName;
    int				RetryAmt;
{
    ClientInfo_t	       *Client;
    struct sockaddr_in		ServerAddr;
    struct timeval		WaitTime;
    struct timeval		Retry;
    struct hostent	       *hp;
    int				Sock = RPC_ANYSOCK;

#ifdef notdef
    if ((Sock = GetResSock()) < 0) {
	Error("Cannot find a reserved socket.");
	return((ClientInfo_t *) NULL);
    }
#endif

    /* Get Server's address */
    memset(ServerAddr.sin_zero, 0, sizeof(ServerAddr.sin_zero));
    ServerAddr.sin_addr.s_addr = inet_addr(ServerName);
    if ((int) (ServerAddr.sin_addr.s_addr) == -1) {
	/* ServerName is not an IP address */
	if ((hp = gethostbyname(ServerName)) == NULL) {
	    Error("No such host as \"%s\".", ServerName);
	    return((ClientInfo_t *) NULL);
	}
	if (hp->h_addrtype != AF_INET) {
	    Error("Bad address type (%d) for \"%s\".", 
		  hp->h_addrtype, ServerName);
	    return((ClientInfo_t *) NULL);
	}
	/* XXX We're only using the first address listed for the server */
	(void) memcpy((char *)&ServerAddr.sin_addr, 
		      hp->h_addr_list[0], hp->h_length);
    }
    ServerAddr.sin_family = AF_INET;
    ServerAddr.sin_port = 0;

    /* Set wait time */
    WaitTime.tv_sec = 10;
    WaitTime.tv_usec = 0;

    Client = CLNTcreate();
    Client->HostName = strdup(ServerName);
    if (Debug) Message("Connecting to SPM server on `%s' . . .", ServerName);
#if	TRANS_TYPE == TRANS_UDP
    Client->Client = (CLIENT *) clntudp_create(&ServerAddr, 
					       (u_long)SPMPROG, 
					       (u_long)SPMVERS,
					       WaitTime, &Sock);
#endif	/* TRANS_UDP */
#if	TRANS_TYPE == TRANS_TCP
    Client->Client = (CLIENT *) clnttcp_create(&ServerAddr, 
					       (u_long)SPMPROG, 
					       (u_long)SPMVERS,
					       &Sock, 0, 0);
#endif	/* TRANS_TCP */

    if (!Client->Client) {
	Error("Cannot make connection to \"%s\" %s.", 
	      ServerName, clnt_spcreateerror(""));
	CLNTdestroy(Client);
	return((ClientInfo_t *) NULL);
    }

    /*
     * Set retry timeout
     */
    if (RetryAmt >= 0)
	Retry.tv_sec = RetryAmt;
    else
	Retry.tv_sec = RPC_RETRY_TIMEOUT;
    Retry.tv_usec = 0;
    if (!clnt_control(Client->Client, CLSET_RETRY_TIMEOUT, (char *)&Retry))
	Error("Set RPC retry timeout failed: %s.", clnt_spcreateerror(""));

    Client->Client->cl_auth = authunix_create_default();

    return(Client);
}

/*
 * Frontend to setup client connection to RPC server
 */
ClientInfo_t *
CLNTconnect(ServerName)
    char		       *ServerName;
{
    static int			First = TRUE;
    ClientInfo_t	       *ClientInfo;
    static struct timeval	TimeOut;

    if (First) {
	/*
	 * Ping the server to insure that inetd starts spmd
	 * and that spmd then has a chance to properly re-register
	 * itself with portmap/rpcbind.
	 */
	ClientInfo = _CLNTconnect(ServerName, RPC_PING_TIMEOUT);
	if (ClientInfo) {
	    TimeOut.tv_sec = RPC_PING_TIMEOUT;
	    TimeOut.tv_usec = 0;
	    (void) clnt_call(ClientInfo->Client, NULLPROC, 
			     xdr_void, (char *)NULL, 
			     xdr_void, (char *)NULL, TimeOut);
	    CLNTdestroy(ClientInfo);
	    First = FALSE;
	} else
	    return(ClientInfo);
    }

    return(_CLNTconnect(ServerName, -1));
}

/*
 * Call RPC proceedure on server.
 */
int
CLNTcall(Client, RPCproc, XDRinproc, InData, XDRoutproc, OutData)
    ClientInfo_t	       *Client;
    u_long			RPCproc;
    xdrproc_t		        XDRinproc;
    caddr_t			InData;
    xdrproc_t		        XDRoutproc;
    caddr_t			OutData;
{
    enum clnt_stat		CLNTstatus;
    static struct timeval	TimeOut;

    TimeOut.tv_sec = RPC_CALL_TIMEOUT;
    TimeOut.tv_usec = 0;
    CLNTstatus = clnt_call(Client->Client, RPCproc, 
			   XDRinproc, InData, 
			   XDRoutproc, OutData, TimeOut);

    if (CLNTstatus != RPC_SUCCESS) {
	Error("RPC call to `%s' failed: %s.", 
	      Client->HostName, clnt_sperrno(CLNTstatus));
	return(-1);
    }

    return(0);
}

/*
 * Generic function to make a SPM RPC call.
 */
int
CLNTspmCall(Client, RPCproc, Query, Result, RetryIntr)
    ClientInfo_t	       *Client;
    u_long			RPCproc;
    SPMquery_t		       *Query;
    SPMresult_t		       *Result;
    int				RetryIntr;
{
    register int		Count = 0;
    int				Status = 0;

    do {
	Status = CLNTcall(Client, RPCproc, 
			  xdr_SPMquery, (caddr_t)Query,
			  xdr_SPMresult, (caddr_t)Result);

	if (Status == 0 && Result->Status == RS_ERROR && RetryIntr <= 0)
	    /* Don't do anything special */
	    break;
	else if (Status == 0 && Result->Status == RS_ERROR) {
	    /* RPC succeeded, but there's a SPM error of some kind */
	    if (Result->Message && Result->Message[0]) {
		/* Retry condition */
		Message("Error from %s: %s", 
			Client->HostName, Result->Message);
		Message("\tRetrying . . .");
		if (Debug) Message("\tSleeping for %d seconds . . .", 
				   SPM_RETRY_INTR);
		sleep(SPM_RETRY_INTR);
	    } else {
		Error("Unknown SPM status code returned by server %s.",
		      Client->HostName);
		Status = -1;
		break;
	    }
	} else if (Status == 0) {
	    /* Success */
	    break;
	} else {
	    /* RPC level error */
	    Status = -1;
	    break;
	}
    } while (Count++ < SPM_MAX_RETRIES);

    return(Status);
}

/*
 * Initialize client info
 */
void
CLNTinit(Ptr)
    ClientInfo_t	       *Ptr;
{
    (void) memset((char *)Ptr, 0, sizeof(ClientInfo_t));
}

/*
 * Create a new client entry
 */
ClientInfo_t *
CLNTcreate()
{
    ClientInfo_t	       *Ptr;

    Ptr = (ClientInfo_t *) xmalloc(sizeof(ClientInfo_t));
    CLNTinit(Ptr);

    return(Ptr);
}

/*
 * Destroy what we created.
 */
void
CLNTdestroy(ClientInfo)
    ClientInfo_t	       *ClientInfo;
{
    if (ClientInfo->Client) {
	if (Debug)
	    Message("Closing connection to `%s'.", ClientInfo->HostName);
	clnt_destroy(ClientInfo->Client);
    }
    if (ClientInfo->HostName)
	(void) free(ClientInfo->HostName);
    if (ClientInfo->ClientName)
	(void) free(ClientInfo->ClientName);
    if (ClientInfo->AddrString)
	(void) free(ClientInfo->AddrString);
    (void) free(ClientInfo);
}
