/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/common.c,v 1.50 1996/03/02 00:39:27 mcooper Exp $";
#endif	/* lint */

/*
 * Functions that are common to all of SPM
 */

#include "defs.h"

/*
 * Common variables
 */
char	       *AgeDir = AGE_DIR; 		/* Directory of age files */
char	       *GecosSep = GECOS_SEP; 		/* GECOS field seperator */
Gecos_t	       *GecosList = NULL;		/* List of GECOS info */
PDitem_t       *PDitemList = NULL;		/* List of PDitem info */
char	       *LocalPwdFile = LOCAL_PWDFILE;	/* Local password file */
char	       *LocalPwdLock = LOCAL_LOCKFILE;	/* Local password lock file */
char	       *LocalShadowFile = SHADOW_PWDFILE;/* Local shadow pwd file */
char	       *LocalShadowLock = SHADOW_LOCKFILE;/* Local shadow pwd lock */
char	       *NISPwdFile = NIS_PWDFILE;	/* Password file */
char	       *NISPwdLock = NIS_LOCKFILE;	/* NIS password lock file */
char	       *NISdir = NIS_DIR;		/* NIS directory */
char	       *PDdir = PD_DIR;			/* Directory of PD files */
char	       *PDserver = NULL;		/* Name of PD server */
char	       *StripSepChars = STRIP_SEP_CHARS;/* Strip seperator chars */
char	       *SPMserver = NULL;		/* Name of SPM server */
char	       *PWHdir = PWH_DIR;		/* Directory of PWH files */
char	       *NISPostCmd = NIS_POSTCMD;	/* NIS Post op command */
char	       *LocalPostCmd = NULL;		/* Local Post op command */
char	       *VerifyList = NULL;		/* List of verifications */
char	       *TransFacList = NULL;		/* List of fac's to TransLog */
char	       *TransDir = TRANS_DIR;		/* List of fac's to TransLog */
AccessInfo_t	PDaccess;			/* PD Access */
AccessInfo_t	SPMaccess;			/* SPM Access */
int		VerifyNum = VERIFY_NUM;		/* Number of verifications */
int		AskTries = MAX_ASK;		/* Ask user how many times? */
int		BackGround = TRUE;		/* Ok to background things */
int		Debug = 0;			/* Enable debugging */
int		Verbose = 1;			/* Be verbose */
int		LockRetryInt = LOCK_RETRY_INT;	/* Lock retry int */
int		MaxLockTries = MAX_LOCK_TRIES;	/* Max lock tries */
int		MaxPWHent = MAX_PWH_ENT;	/* Max no. of pwd's kept */
int		MaxGecosSize = 0;		/* Max pw_gecos size */
int		MaxShellSize = MAX_FLD_SHELL;	/* Max pw_shell size */
int		MaxPwdSize = MAX_FLD_PWD;	/* Max pw_pwd size */
int		UseSyslog = 0;			/* Msgs to syslog */
int		DoKeys = 0;			/* Show keywords */
int		ExitOnError = 0;		/* Exit on error flag */

/*
 * Configuration file variables.
 */
CONFvartab_t VarTable[] = {
    { "AgeDir",		VT_STRING,	(Opaque) &AgeDir },
    { "AskTries",	VT_INT,		(Opaque) &AskTries },
    { "GecosSep",	VT_STRING,	(Opaque) &GecosSep },
    { "LocalPostCmd",	VT_STRING,	(Opaque) &LocalPostCmd },
    { "LocalPwdFile",	VT_STRING,	(Opaque) &LocalPwdFile },
    { "LocalPwdLock",	VT_STRING,	(Opaque) &LocalPwdLock },
    { "LocalShadowFile",VT_STRING,	(Opaque) &LocalShadowFile },
    { "LocalShadowLock",VT_STRING,	(Opaque) &LocalShadowLock },
    { "LockRetryInt",	VT_INT,		(Opaque) &LockRetryInt },
    { "MaxPWHent",	VT_INT,		(Opaque) &MaxPWHent },
    { "MaxPwdSize",	VT_INT,		(Opaque) &MaxPwdSize },
    { "MaxShellSize",	VT_INT,		(Opaque) &MaxShellSize },
    { "NISPostCmd",	VT_STRING,	(Opaque) &NISPostCmd },
    { "NISPwdFile",	VT_STRING,	(Opaque) &NISPwdFile },
    { "NISPwdLock",	VT_STRING,	(Opaque) &NISPwdLock },
    { "NISdir",		VT_STRING,	(Opaque) &NISdir },
    { "PDaccess",	VT_DATA,	(Opaque) &PDaccess, 	ACCESSset },
    { "PDserver",	VT_STRING,	(Opaque) &PDserver },
    { "SPMaccess",	VT_DATA,	(Opaque) &SPMaccess,	ACCESSset },
    { "StripSepChars",	VT_STRING,	(Opaque) &StripSepChars },
    { "SPMserver",	VT_STRING,	(Opaque) &SPMserver },
    { "PdDir",		VT_STRING,	(Opaque) &PDdir },
    { "PwdHistDir",	VT_STRING,	(Opaque) &PWHdir },
    { "TransDir",	VT_STRING,	(Opaque) &TransDir },
    { "TransFacList",	VT_STRING,	(Opaque) &TransFacList },
    { "VerifyList",	VT_STRING,	(Opaque) &VerifyList },
    { "VerifyNum",	VT_INT,		(Opaque) &VerifyNum },
    { NULL },
};

/*
 * This is the default table used for GECOS field information.
 * The "GECOS" keyword in the spm.conf file can be used to override
 * these values.
 */
Gecos_t DefaultGecosTable[] = {
    { "Full Name",		"John Q. Smith",	40 },
    { "Office Location",	"SAL 125",		20 },
    { "Office Phone Number",	"740-2957",		10 },
    { "Home Phone Number",	"213-777-3456",		10 },
    { "Comments",		"My work account",	20 },
    { 0 },
};

/*
 * Default Personel Data Item types.
 */
PDitem_t DefaultPDitems[] = {
    { "SSN",	"Social Security Number",	"123-45-6789", PDF_NOECHO },
    { "DOB",	"Date of Birth",		"1/2/95" },
    { "MomName","Mother's Maiden Name",		"Thompson" },
    { "HomeZip","Home ZIP Code",		"90278" },
    { 0 },
};

/*
 * PDitem Flag lookup table
 */
NameKey_t PDitemFlagTable[] = {
    { "noecho",		PDF_NOECHO },
    { "stripsep",	PDF_STRIPSEP },
    { 0 },
};

/*
 * Get the crypted version of String.
 */
char *
getcrypt(String)
    char		       *String;
{
    char 			saltc[2];
    time_t 			salt;
    register int		i;
    register int		c;

    (void) time(&salt);
    salt = 9 * getpid();
    saltc[0] = salt & 077;
    saltc[1] = (salt>>6) & 077;
    for (i = 0; i < 2; i++) {
	c = saltc[i] + '.';
	if (c > '9')
	    c += 7;
	if (c > 'Z')
	    c += 6;
	saltc[i] = c;
    }

    return(crypt(String, saltc));
}

/*
 * Format a message.
 */
#if	ARG_TYPE == ARG_VARARGS
char *
Format(va_alist)
    va_dcl
{
    static char			Buff[BUFSIZ];
    va_list			Args;
    char		       *Fmt;

    va_start(Args);
    Fmt = va_arg(Args, char *);
    (void) vsprintf(Buff, Fmt, Args);
    va_end(Args);

    return(Buff);
}
#endif	/* ARG_VARARGS */
#if	ARG_TYPE == ARG_STDARG
char *
Format(char *Fmt, ...)
{
    static char			Buff[BUFSIZ];
    va_list			Args;

    va_start(Args, Fmt);
    (void) vsprintf(Buff, Fmt, Args);
    va_end(Args);

    return(Buff);
}
#endif	/* ARG_STDARG */

/*
 * Print an error message.
 */
#if	ARG_TYPE == ARG_VARARGS
void Error(va_alist)
    va_dcl
{
    va_list			Args;
    char		       *Fmt;

    va_start(Args);
    Fmt = va_arg(Args, char *);
    if (UseSyslog)
	(void) vsyslog(LOG_ERR, Fmt, Args);
    if (!UseSyslog || Debug) {
	(void) fprintf(stderr, "%s: ", ProgramName);
	(void) vfprintf(stderr, Fmt, Args);
	(void) fprintf(stderr, "\n");
    }

    va_end(Args);
}
#endif	/* ARG_TYPE == ARG_VARARGS */
#if	ARG_TYPE == ARG_STDARG
void Error(char *Fmt, ...)
{
    va_list			Args;
    extern char		       *ProgramName;

    va_start(Args, Fmt);
    if (UseSyslog)
	(void) vsyslog(LOG_ERR, Fmt, Args);
    if (!UseSyslog || Debug) {
	(void) fprintf(stderr, "%s: ", ProgramName);
	(void) vfprintf(stderr, Fmt, Args);
	(void) fprintf(stderr, "\n");
    }
    va_end(Args);
}
#endif	/* ARG_TYPE == ARG_STDARG */

/*
 * Print a normal message.
 */
#if	ARG_TYPE == ARG_VARARGS
void Message(va_alist)
     va_dcl
{
    va_list			Args;
    char		       *Fmt;

    va_start(Args);

    Fmt = va_arg(Args, char *);
    if (UseSyslog)
	(void) vsyslog(LOG_INFO, Fmt, Args);
    if (!UseSyslog || Debug) {
	(void) vfprintf(stderr, Fmt, Args);
	(void) fprintf(stderr, "\n");
    }

    va_end(Args);
}
#endif	/* ARG_TYPE == ARG_VARARGS */
#if	ARG_TYPE == ARG_STDARG
void Message(char *Fmt, ...)
{
    va_list			Args;
    extern char		       *ProgramName;

    va_start(Args, Fmt);
    if (UseSyslog)
	(void) vsyslog(LOG_INFO, Fmt, Args);
    if (!UseSyslog || Debug) {
	(void) vfprintf(stderr, Fmt, Args);
	(void) fprintf(stderr, "\n");
    }
    va_end(Args);
}
#endif	/* ARG_TYPE == ARG_STDARG */

/*
 * Frontend to malloc()
 */
extern void *xmalloc(Amt)
    int				Amt;
{
    void		       *Ptr;

    Ptr = (void *) malloc(Amt);
    if (!Ptr) {
	Error("Unable to malloc %d bytes.", Amt);
	exit(2);
    }

    return(Ptr);
}

/*
 * Frontend to calloc()
 */
extern void *xcalloc(NumEle, Amt)
    int				NumEle;
    size_t			Amt;
{
    void		       *Ptr;

    Ptr = (void *) calloc(NumEle, Amt);
    if (!Ptr) {
	Error("Unable to calloc %d*%d bytes.", NumEle, Amt);
	exit(2);
    }

    return(Ptr);
}

/*
 * Get reserved socket.
 */
extern int
GetResSock()
{
    int				Port = IPPORT_RESERVED - 1;
    int				Sock;

    if ((Sock = rresvport(&Port)) < 0)
	Error("Get reserved port failed: %s.", SYSERR);

    return(Sock);
}

/*
 * Get Time String
 */
char *
GetTimeStr(TimeVal)
    time_t			TimeVal;
{
    register char	       *TimeStr = NULL;
    register char	       *cp;

    if (TimeVal == 0)
	return("(unknown)");

    if (TimeStr = ctime(&TimeVal))
	if (cp = strchr(TimeStr, '\n'))
	    *cp = CNULL;

    return(TimeStr);
}

/*
 * Get our domain name.
 */
char *
GetDomain(Facility)
    Facility_t		       *Facility;
{
    if (Facility && Facility->GetDomain)
	return((*Facility->GetDomain)());
    else
	return(DOMAIN_DEFAULT);
}

/*
 * Check Domain for validity.
 * If not set, determine domain to use.
 */
char *
CheckDomain(Domain, ErrBuff)
    char		       *Domain;
    char		       *ErrBuff;
{
    char		       *Name = NULL;

    if (Domain && Domain[0]) {
	/*
	 * Don't allow '/' in domain name for security reasons.
	 */
	if (strchr(Domain, '/')) {
	    (void) sprintf(ErrBuff, 
			   "Domain name \"%s\" contains illegal characters.",
			   Domain);
	    return((char *) NULL);
	}
	Name = Domain;
    }

    if (!Name)
	Name = DOMAIN_DEFAULT;

    return(Name);
}

#if	!defined(HAVE_XDR_TIME_T)
/*
 * XDR time_t function
 */
int
xdr_time_t(XDRptr, ulPtr)
    register XDR	       *XDRptr;
    u_long		       *ulPtr;
{
    return(xdr_u_long(XDRptr, ulPtr));
}
#endif	/* HAVE_XDR_TIME_T */

/*
 * Convert an int to an ascii string.
 */
char *
itoa(Value)
    int				Value;
{
    static char			Buff[100];

    (void) sprintf(Buff, "%d", Value);

    return(Buff);
}

/*
 * Convert a long to an ascii string.
 */
char *
ltoa(Value)
    long			Value;
{
    static char			Buff[100];

    (void) sprintf(Buff, "%d", Value);

    return(Buff);
}

/*
 * Convert an unsigned long to an ascii string.
 */
char *
ultoa(Value)
    unsigned long		Value;
{
    static char			Buff[100];

    (void) sprintf(Buff, "%d", Value);

    return(Buff);
}

/*
 * Get base name of String.
 */
char *
Basename(String, Ch)
    char		       *String;
    char			Ch;
{
    register char	       *cp;

    cp = strrchr(String, Ch);
    if (cp)
	++cp;
    else
	cp = String;

    return(cp);
}

/*
 * Get PCtype name for Type.
 */
char *
GetPCname(Type)
    PCtype_t			Type;
{
    switch (Type) {
    case PC_PWD:	return("password");
    case PC_GECOS:	return("finger (GECOS)");
    case PC_SHELL:	return("login shell");
    default:		return("<UNKNOWN>");
    }
}

/*
 * Get PCtype_t for TypeStr
 */
PCtype_t
GetPCtype(TypeStr)
    char		       *TypeStr;
{
    if (EQ(PCS_PWD, TypeStr))
	return(PC_PWD);
    else if (EQ(PCS_GECOS, TypeStr))
	return(PC_GECOS);
    else if (EQ(PCS_SHELL, TypeStr))
	return(PC_SHELL);
    else
	return(PC_UNKNOWN);
}

/*
 * Get our host name.
 */
#if	HOSTNAME_TYPE == HOSTNAME_UNAME
#include <sys/utsname.h>
char *
GetHostName()
{
    static struct utsname	un;

    if (uname(&un) < 0)
	return((char *) NULL);

    return(un.nodename);
}
#endif	/* HOSTNAME_UNAME */
#if	HOSTNAME_TYPE == HOSTNAME_GETHOSTNAME
char *
GetHostName()
{
    static char			Buff[256];

    if (gethostname(Buff, sizeof(Buff)) < 0)
	return((char *) NULL);

    return(Buff);
}
#endif	/* HOSTNAME_UNAME */

/*
 * Get an error string from a result.
 */
char *
ResultErrStr(Result)
    SPMresult_t		       *Result;
{
    static char			Buff[100];

    switch(Result->Status) {
    case RS_OK:
	return("Success");

    case RS_NOENT:
	return("No such entry");

    case RS_NOMATCH:
	return("No match");

    case RS_ERROR:
	if (Result->Message && Result->Message[0])
	    return(Result->Message);
	else
	    return("Unknown error");

    default:
	(void) sprintf(Buff, "Unknown error number %d", Result->Status);
	return(Buff);
    }
}

/*
 * Get name of SPM server
 */
char *
GetSPMserver(UserName, Domain)
    char		       *UserName;
    char		       *Domain;
{
    Facility_t		       *Facility;

    Facility = FACget(UserName, FAC_UNKNOWN);
    if (!Facility) {
	Error("Could not determine name of your SPM server.");
	exit(1);
    }
    if (Facility->GetServer)
	return( (*Facility->GetServer)(Domain) );

    return((char *) NULL);
}

/*
 * Initialize a SPMquery_t.
 */
void
QueryInit(QueryPtr)
    SPMquery_t		       *QueryPtr;
{
    (void) memset((char *)QueryPtr, 0, sizeof(SPMquery_t));
    QueryPtr->UserName = "";
    QueryPtr->ReqUser = "";
    QueryPtr->CurPass = "";
    QueryPtr->Arg = "";
    QueryPtr->Facility = "";
    QueryPtr->Domain = "";
}

/*
 * Initialize a SPMresult_t.
 */
void
ResultInit(Ptr)
    SPMresult_t		       *Ptr;
{
    (void) memset((char *)Ptr, 0, sizeof(SPMresult_t));
    Ptr->Status = RS_ERROR;
    Ptr->Message = "";
}

/*
 * Get keyword from a string.
 * String looks like 'key = value'.
 */
char *
GetKey(String, ErrBuf)
    char		       *String;
    char		       *ErrBuf;
{
    register char	       *cp;
    char		       *Name;
    char		       *End;
    static char			Buff[BUFSIZ];

    if (strlen(String) > sizeof(Buff)-1) {
	if (ErrBuf) (void) sprintf(ErrBuf, "GetKey() Buffer size too small");
	return((char *) NULL);
    }
    (void) strcpy(Buff, String);

    End = strchr(Buff, '=');
    if (!End) {
	if (ErrBuf) (void) sprintf(ErrBuf, "No '=' found");
	return((char *) NULL);
    }
    *End = NULL;

    /* Find start of keyword - Skip leading white space */
    for (cp = Buff; cp && *cp && isspace(*cp); ++cp);
    Name = cp;
    /* Find end of keyword */
    for (cp = Name; cp && *cp && !isspace(*cp); ++cp);
    *cp = CNULL;

    return(Name);
}

/*
 * Get value from a string.
 * String looks like 'key = value'.
 * String may be modified.
 */
char *
GetValue(String, ErrBuf)
    char		       *String;
    char		       *ErrBuf;
{
    register char	       *cp;
    char		       *Name;
    char		       *End;

    Name = strchr(String, '=');
    if (Name && *Name)
	++Name;
    else {
	if (ErrBuf) (void) sprintf(ErrBuf, "No '=' found");
	return((char *) NULL);
    }

    /* Find start of value - Skip leading white space */
    for (cp = Name; cp && *cp && isspace(*cp); ++cp);
    Name = cp;
    /* Remove trailing white space */
    End = &Name[strlen(Name)-1];
    for (cp = End; cp && *cp && isspace(*cp); --cp);
    if (cp != End && *cp)
	*++cp = CNULL;

    return(Name);
}

/*
 * Make directory DirName.
 * If Recurse is true, then recursively make directories as needed.
 */
int
MakeDir(DirName, Mode, Recurse, ErrBuff)
    char		       *DirName;
    mode_t			Mode;
    int				Recurse;
    char		       *ErrBuff;
{
    struct stat			StatBuf;
    register char	       *cp;
    char		       *SavePtr = NULL;
    char		       *Name;
    int				Status;

    if (!Recurse) 
	if (mkdir(DirName, Mode) != 0) {
	    if (ErrBuff)
		sprintf(ErrBuff, "mkdir %s failed: %s", DirName, SYSERR);
	    return(-1);
	} else 
	    return(0);

    for (cp = Name = DirName; cp; Name = cp+1) {
	if (SavePtr) *SavePtr = '/';
	if ((cp == Name) && (*cp == '/'))
	    ++Name;
	cp = strchr(Name, '/');
	if (cp) {
	    SavePtr = cp;
	    *cp = CNULL;
	}
	Status = stat(DirName, &StatBuf);
	if (Status == 0)
	    continue;
	if (errno == ENOENT) {
	    if (mkdir(DirName, Mode) != 0) {
		if (ErrBuff)
		    sprintf(ErrBuff, "mkdir %s failed: %s", DirName, SYSERR);
		if (SavePtr) *SavePtr = '/';
		return(-1);
	    }
	} else {
	    if (ErrBuff)
		sprintf(ErrBuff, "stat %s failed: %s", DirName, SYSERR);
	    if (SavePtr) *SavePtr = '/';
	    return(-1);
	}
    }

    return(0);
}

/*
 * Check for directory named DirName.
 * If DirName does not exist, create it.
 * Returns 1 if directory is created.
 * Returns 0 if no action is taken.
 * Returns -1 on error.
 */
int
CheckDir(DirName)
    char		       *DirName;
{
    static char			ErrBuff[BUFSIZ];
    struct stat			StatBuf;

    if (stat(DirName, &StatBuf) != 0)
	if (MakeDir(DirName, DIR_MODE, 1, ErrBuff) != 0) {
	    Error("Cannot create %s: %s.", DirName, ErrBuff);
	    return(-1);
	} else {
	    Message("Created directory `%s'", DirName);
	    return(1);
	}

    if (!S_ISDIR(StatBuf.st_mode)) {
	Error("Path name %s exists but is not a directory.", DirName);
	return(-1);
    }

    return(0);
}

/*
 * Lock the file associated with descriptor Desc.
 */
FILElock(Desc, LockType, File, ErrBuff)
    int				Desc;
    int				LockType;
    char		       *File;
    char		       *ErrBuff;
{
    static struct flock		flock;

    (void) memset((char *)&flock, 0, sizeof(flock));
    flock.l_type = LockType;
    flock.l_whence = SEEK_SET;

    if (fcntl(Desc, F_SETLK, &flock) == -1) {
	(void) sprintf(ErrBuff, "Set lock failed: %s: %s", File, SYSERR);
	return(-1);
    }

    if (Debug)
	Message("Locked file %s", File);

    return(0);
}

/*
 * Unlock the file associated with descriptor Desc.
 */
FILEunlock(Desc, File, ErrBuff)
    int				Desc;
    char		       *File;
    char		       *ErrBuff;
{
    static struct flock		flock;

    (void) memset((char *) &flock, 0, sizeof(flock));
    flock.l_type = F_UNLCK;
    flock.l_whence = SEEK_SET;

    if (fcntl(Desc, F_SETLK, &flock) == -1) {
	if (ErrBuff)
	    (void) sprintf(ErrBuff, "Unlock failed: %s: %s", File, SYSERR);
	return(-1);
    }

    if (Debug)
	Message("Unlocked file %s", File);

    return(0);
}

/*
 * Get program version info
 */
char *
GetVersion()
{
    extern char			BuildDate[];
    extern char			BuildHost[];
    extern char			BuildUser[];
    static char			Buff[BUFSIZ];

    (void) sprintf(Buff, "SPM version ");
    if (PATCHLEVEL)
	(void) sprintf(Buff + strlen(Buff), "%s.%d", VERSION_STR, PATCHLEVEL);
    else
	(void) sprintf(Buff + strlen(Buff), "%s", VERSION_STR);

    (void) sprintf(Buff + strlen(Buff), " (%s)\nBuilt %s on %s", 
		   VERSION_STATUS, BuildDate, BuildHost);
    if (BuildUser[0])
	(void) sprintf(Buff + strlen(Buff), " by %s", BuildUser);

    (void) sprintf(Buff + strlen(Buff), "\n\n%s", VERSION_COPYRIGHT);

    return(Buff);
}

/*
 * Trim leading and trailing white space
 */
char *TrimWhite(String)
    char		       *String;
{
    register char	       *cp;
    char		       *NewString;
    char		       *End;

    for (NewString = String; isspace(*NewString); ++NewString);

    cp = End = &NewString[strlen(NewString) - 1];

    while (isspace(*cp))
	--cp;

    if (cp != End)
	*(cp+1) = CNULL;

    return(NewString);
}

#if	TERMIO_TYPE == TERMIO_TERMIOS

#include <termios.h>
#include <unistd.h>

static struct termios		TermAttr;

/*
 * Turn off terminal echoing
 */
int
TTYechoOff()
{
    if (tcgetattr(STDIN_FILENO, &TermAttr) != 0) {
	Error("Cannot get terminal attributes - Echoing will remain enabled.");
	return(-1);
    }

    TermAttr.c_lflag &= ~(ECHO);

    if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &TermAttr) != 0) {
	Error("Cannot set terminal attributes - Echoing will remain enabled.");
	return(-1);
    }

    return(0);
}

/*
 * Turn on Terminal echo'ing
 */
int
TTYechoOn()
{
    if (!TermAttr.c_lflag)
	/* We're not initialized (TTYechoOff() was never called) */
	return(0);

    TermAttr.c_lflag |= ECHO;
    if (tcsetattr(STDIN_FILENO, TCSANOW, &TermAttr) != 0) {
	Error(
"Cannot restore terminal attributes - Echoing will remain disabled.");
	return(-1);
    }

    return(0);
}
#endif	/* TERMIO_TYPE == TERMIO_TERMIOS */
