/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/conf.c,v 1.27 1996/03/02 00:39:38 mcooper Exp $";
#endif	/* lint */

/*
 * Configuration functions
 */

#include "defs.h"
#include <ctype.h>

static char *			CONFset();
static char *			CONFgecos();
static char *			CONFpditem();
static char *			CONFinclude();

/*
 * Key Table
 */
CONFkeytab_t KeyTable[] = {
    { "gecos",		CONFgecos },
    { "pditem",		CONFpditem },
    { "set",		CONFset },
    { "include",	CONFinclude },
    { NULL },
};

/*
 * Find key table entry for key.
 */
CONFkeytab_t *
CONFgetkeytab(Key, Size)
    char		       *Key;
    size_t			Size;
{
    register int		i;

    for (i = 0; KeyTable[i].Key; ++i)
	if (strncasecmp(KeyTable[i].Key, Key, Size) == 0)
	    return(&KeyTable[i]);

    return((CONFkeytab_t *) NULL);
}

/*
 * Find variable table entry for Name.
 */
CONFvartab_t *
CONFgetvartab(Name, Table)
    char		       *Name;
    CONFvartab_t	       *Table;
{
    register CONFvartab_t      *Ptr;

    for (Ptr = Table; Ptr && Ptr->Name; ++Ptr)
	if (strcasecmp(Ptr->Name, Name) == 0)
	    return(Ptr);

    return((CONFvartab_t *) NULL);
}

/*
 * Install any changes that are needed after both the config
 * file and then the command line args are parsed.
 */
void
CONFinstall()
{
    Facility_t		       *Facility;
    register Gecos_t	       *Gecos;

    /*
     * Update facility entries.
     */
    if (NISPwdFile || NISPwdLock) {
	Facility = FAClookup(FN_NIS, FALSE);
	if (!Facility)
	    exit(2);
	if (NISPwdFile)
	    Facility->PwdFile = NISPwdFile;
	if (NISPwdLock)
	    Facility->PwdLock = NISPwdLock;
    }
    if (LocalPwdFile || LocalPwdLock) {
	Facility = FAClookup(FN_LOCAL, FALSE);
	if (!Facility)
	    exit(2);
	if (LocalPwdFile)
	    Facility->PwdFile = LocalPwdFile;
	if (LocalPwdLock)
	    Facility->PwdLock = LocalPwdLock;
    }
    if (LocalShadowFile || LocalShadowLock) {
	Facility = FAClookup(FN_LOCAL, FALSE);
	if (!Facility)
	    exit(2);
	if (LocalShadowFile)
	    Facility->ShadowFile = LocalShadowFile;
	if (LocalShadowLock)
	    Facility->ShadowLock = LocalShadowLock;
    }
    if (NISPostCmd) {
	Facility = FAClookup(FN_NIS, FALSE);
	if (!Facility)
	    exit(2);
	Facility->PostCmd = NISPostCmd;
    }
    if (LocalPostCmd) {
	Facility = FAClookup(FN_LOCAL, FALSE);
	if (!Facility)
	    exit(2);
	Facility->PostCmd = LocalPostCmd;
    }

    if (!PDitemList) {
	/*
	 * If no PDitemList was set in spm.conf, initialize a default
	 */
	register PDitem_t      *Last;
	register int		i;
	extern PDitem_t		DefaultPDitems[];

	Last = PDitemList = &DefaultPDitems[0];
	for (i = 1; DefaultPDitems[i].Key; ++i) {
	    Last->Next = &DefaultPDitems[i];
	    Last = Last->Next;
	}
    }

    if (!GecosList) {
	/*
	 * No GecosList was set in spm.conf so link together the
	 * builtin table and use that.
	 */
	register int		i;
	register Gecos_t       *Last;
	extern Gecos_t	        DefaultGecosTable[];

	Last = GecosList = &DefaultGecosTable[0];
	for (i = 1; DefaultGecosTable[i].Label; ++i) {
	    Last->Next = &DefaultGecosTable[i];
	    Last = Last->Next;
	}
    }

    /*
     * Figure out maximum size of pw_gecos
     */
    for (MaxGecosSize = 0, Gecos = GecosList; Gecos; Gecos = Gecos->Next)
	MaxGecosSize += Gecos->MaxSize;
}

/*
 * Do initialization necessary before config begins.
 */
void CONFinit()
{
    /*
     * Clear global vars we may set that are uninitialized.
     */
    (void) memset((char *)&PDaccess, CNULL, sizeof(PDaccess));
    (void) memset((char *)&SPMaccess, CNULL, sizeof(SPMaccess));
    PDaccess.Name = "PDaccess";
    SPMaccess.Name = "SPMaccess";
}

/*
 * Parse a configuration file.
 */
int
_CONFparse(File, FilePtr, VarTab)
    char		       *File;
    FILE		       *FilePtr;
    CONFvartab_t	       *VarTab;
{
    static char			Buff[BUFSIZ];
    register char	       *cp;
    int				Status = 0;
    register int		LineNo = 0;
    CONFkeytab_t	       *KeyTab;
    char		       *ErrStr;

    if (Debug) Message("Reading `%s' . . .", File);

    while (fgets(Buff, sizeof(Buff), FilePtr)) {
	++LineNo;
	if (cp = strchr(Buff, '\n'))
	    *cp = CNULL;
	if (!Buff[0] || Buff[0] == '#')
	    continue;

	/* Find end of word */
	for (cp = Buff; cp && *cp && !isspace(*cp); ++cp);

	if (KeyTab = CONFgetkeytab(Buff, (size_t) (cp - Buff)) ) {
	    /* Skip leading white space */
	    for ( ; cp && isspace(*cp); ++cp);
	    ErrStr = (*KeyTab->Func)(cp, VarTab);
	    if (ErrStr) {
		Error("%s: line %d: Error", File, LineNo);
		if (*ErrStr)
		    Error(" - %s.", ErrStr);
		++Status;
	    }
	} else {
	    Error("%s: line %d: Syntax error - \"%.*s\" unknown keyword.", 
		  File, LineNo, cp - Buff, Buff);
	    ++Status;
	}
    }

    return(Status);
}

/*
 * Front end to parse config file.
 */
int
CONFparse(FileList, VarTab)
    char		       *FileList;
    CONFvartab_t	       *VarTab;
{
    FILE		       *fp = NULL;
    int				Status = 0;
    static char		      **Argv = NULL;
    static int			Argc = 0;
    register int		i;

    /*
     * Parse the first config file we can open.
     */
    Argc = StrToArgv(FileList, ":", &Argv, SF_RMWS);
    for (i = 0; i < Argc && !fp; ++i) {
	fp = fopen(Argv[i], "r");
	if (fp) {
	    if (_CONFparse(Argv[i], fp, VarTab) < 0)
		Status = -1;
	    (void) fclose(fp);
	} else
	    if (Debug) Message("Cannot open %s: %s.", Argv[i], SYSERR);
    }

    if (Argv)
	DestroyArgv(&Argv, Argc);

    return(Status);
}

/*
 * String should be of form 'foo = bar'.
 * Return NULL on success.
 * Return error string on error.
 */
static char *
CONFset(String, VarTab)
    char		       *String;
    CONFvartab_t	       *VarTab;
{
    static char			Buff[BUFSIZ];
    register char	       *Name;
    register char	       *Value;
    CONFvartab_t	       *VarTabPtr;

    if (!strchr(String, '='))
	return("no ``='' found");

    /*
     * Find the keyword name.
     */
    Name = GetKey(String, Buff);
    if (!Name)
	return(Buff);
    VarTabPtr = CONFgetvartab(Name, VarTab);
    if (!VarTabPtr) {
	(void) sprintf(Buff, "\"%s\" unknown variable", Name);
	return(Buff);
    }

    Value = GetValue(String, Buff);
    if (!Value)
	return(Buff);

    if (VarTabPtr->Variable) {
	switch (VarTabPtr->Type) {
	case VT_STRING:
	    *(char **)(VarTabPtr->Variable) = strdup(Value);
	    break;

	case VT_INT:
	    *(int *)(VarTabPtr->Variable) = strtol(Value, (char **)NULL, 0);
	    break;
	}
    }
    if (VarTabPtr->Set)
	if ((*VarTabPtr->Set)(Name, Value, VarTabPtr->Variable, Buff) < 0)
	    return(Buff);

    return((char *) NULL);
}

/*
 * String should be of form "label:example:maxsize".
 * Return NULL on success.
 * Return error string on error.
 */
static char *
CONFgecos(String, VarTab)
    char		       *String;
    CONFvartab_t	       *VarTab;
{
    register char	       *cp;
    register Gecos_t	       *GecosPtr;
    Gecos_t		       *Gecos;
    char		       *StrCopy;

    if (!strchr(String, ':'))
	return("no ``:'' found");

    Gecos = (Gecos_t *) xcalloc(1, sizeof(Gecos_t));
    StrCopy = strdup(String);

    cp = strtok(StrCopy, ":");
    if (!cp)
	return("Field 2 missing");
    Gecos->Label = TrimWhite(cp);

    cp = strtok(NULL, ":");
    if (!cp)
	return("Field 3 missing");
    Gecos->Example = TrimWhite(cp);

    cp = strtok(NULL, ":");
    if (!cp)
	return("Field 4 missing");
    Gecos->MaxSize = atoi(TrimWhite(cp));

    if (GecosList) {
	for (GecosPtr = GecosList; GecosPtr && GecosPtr->Next; 
	     GecosPtr = GecosPtr->Next);
	GecosPtr->Next = Gecos;
    } else
	GecosList = Gecos;

    return((char *) NULL);
}

/*
 * String should be of form "Key:Description:Example:Flags"
 * Flags is a comma seperated list.
 *
 * Return NULL on success.
 * Return error string on error.
 */
static char *
CONFpditem(String, VarTab)
    char		       *String;
    CONFvartab_t	       *VarTab;
{
    register char	       *cp;
    register char	       *cpf;
    register Gecos_t	       *GecosPtr;
    PDitem_t		       *Item;
    char		       *StrCopy;

    if (!strchr(String, ':'))
	return("no ``:'' found");

    Item = (PDitem_t *) xcalloc(1, sizeof(PDitem_t));
    StrCopy = strdup(String);

    cp = strtok(StrCopy, ":");
    if (!cp)
	return("Field 2 missing");
    Item->Key = TrimWhite(cp);

    cp = strtok(NULL, ":");
    if (!cp)
	return("Field 3 missing");
    Item->Desc = TrimWhite(cp);

    cp = strtok(NULL, ":");
    if (!cp)
	return("Field 4 missing");
    Item->Example = TrimWhite(cp);

    /*
     * Flags field is optional
     */
    cp = strtok(NULL, ":");
    if (cp)
	for (cpf = strtok(cp, ","); cpf; cpf = strtok(NULL, ","))
	    Item->Flags |= PDgetItemFlag(cpf);

    PDitemAddList(&PDitemList, Item);

    return((char *) NULL);
}

/*
 * String should be name of a file to read.
 * Return NULL on success.
 * Return error string on error.
 */
static char *
CONFinclude(String, VarTab)
    char		       *String;
    CONFvartab_t	       *VarTab;
{
    if (CONFparse(String, VarTab) != 0)
	return("");

    return((char *) NULL);
}
