/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/dbinfo.c,v 1.25 1996/03/02 00:40:57 mcooper Exp $";
#endif	/* lint */

/*
 * Database information.
 */

#include "defs.h"

static int			DBshowAGE();
static int			DBshowPD();
static int			DBshowPWH();

DBtable_t DBtable[] = {
    { "pd",	"Get Personel Data",
	  DBshowPD,	PDread,		PDwrite, 	PDset,	PDdelete },
    { "pwh",	"Get Password History",
	  DBshowPWH,	PWHread,	PWHwrite, 	PWHset,	PWHdelete },
    { "age",	"Get Password Age Data",
	  DBshowAGE,	AGEread,	AGEwrite,	AGEset,	AGEdelete },
    { NULL },
};

/*
 * Get an entry named Name from the DB table.
 */
DBtable_t *
DBgettable(Name)
    char		       *Name;
{
    register int		i;

    for (i = 0; DBtable[i].Type; ++i)
	if (strcasecmp(Name, DBtable[i].Type) == 0)
	    return(&DBtable[i]);

    return((DBtable_t *) NULL);
}

/*
 * List all databases.
 */
void
DBlist()
{
    register int i;

    Error("The following are the valid databases:");
    Error("");
    Error("\t%-10s %s", "Database", "Description");
    Error("\t%-10s %s", "--------", "-----------");
    for (i = 0; DBtable[i].Type; ++i)
	Error("\t%-10s %s", DBtable[i].Type, DBtable[i].Desc);
    Error("");
}

/*
 * Query AGE database and show results.
 */
static int
DBshowAGE(UserName, DomainName)
    char		       *UserName;
    char		       *DomainName;
{
    AGEinfo_t		       *Info;

    Info = AGEgetbyuser(UserName, DomainName);
    if (!Info || Info->Result.Status != RS_OK) {
	Error("Lookup \"%s\" failed: %s", 
	      UserName, ResultErrStr(&Info->Result));
	return(-1);
    }

    AGEshow(UserName, Info);

    return(0);
}

/*
 * Query PD database and show results.
 */
static int
DBshowPD(UserName, DomainName)
    char		       *UserName;
    char		       *DomainName;
{
    static char			Buff[BUFSIZ];
    PDinfo2_t		       *Info;
    char		       *Domain;

    Domain = CheckDomain(DomainName, Buff);
    if (!Domain) {
	Error("%s", Buff);
	return(-1);
    }

    Info = PDgetbyuser(UserName, Domain);
    if (!Info || Info->Result.Status != RS_OK) {
	Error("Lookup \"%s\" failed: %s", 
	      UserName, ResultErrStr(&Info->Result));
	return(-1);
    }

    PDshow(UserName, Info);

    return(0);
}

/*
 * Query PWH database and show results.
 */
static int
DBshowPWH(UserName, DomainName)
    char		       *UserName;
    char		       *DomainName;
{
    static char			Buff[BUFSIZ];
    PwdHist_t		       *Info;
    char		       *Domain;

    Domain = CheckDomain(DomainName, Buff);
    if (!Domain) {
	Error("%s", Buff);
	return(-1);
    }
	
    Info = PWHgetbyuser(UserName, Domain);
    if (Info->Result.Status != RS_OK) {
	Error("Lookup \"%s\" failed: %s", 
	      UserName, ResultErrStr(&Info->Result));
	return(-1);
    }

    PWHshow(UserName, Info);

    return(0);
}

/*
 * Check to see if Pass has been used before.
 * Return DB_TRUE if it has been.
 * Return DB_FALSE if it has not been.
 * Return DB_ERROR on error.
 */
DBstatus_t
DBcheckpwd(QueryInfo, NewPass)
    SPMquery_t		       *QueryInfo;
    char		       *NewPass;
{
    static char			Failed[] = "Check password history failed";
    static SPMquery_t		Query;
    static SPMresult_t		Result;
    ClientInfo_t	       *Client;

    if (!NewPass) {
	Error("Invalid parameter");
	return(DB_ERROR);
    }

    if (!SPMserver) 
	if (!(SPMserver = GetSPMserver(QueryInfo->UserName, 
				       QueryInfo->Domain))) {
	    Error("No SPM server defined.");
	    return(DB_ERROR);
	}

    if (!(Client = CLNTconnect(SPMserver)))
	return(DB_ERROR);

    /*
     * Form SPM query
     */
    memcpy((char *)&Query, (char *)QueryInfo, sizeof(SPMquery_t));
    if (!Query.Domain)
	Query.Domain = GetDomain(NULL);
    Query.Arg = NewPass;
    memset((char *)&Result, 0, sizeof(Result));

    if (CLNTcall(Client, SPMPROC_CHECKPWDHIST,
		 xdr_SPMquery, (caddr_t)&Query,
		 xdr_SPMresult, (caddr_t)&Result) < 0) {
	Error("%s", Failed);
	return(DB_ERROR);
    }

    CLNTdestroy(Client);

    switch (Result.Status) {
    case RS_NOENT:
	if (Debug) Message("No pwd history entry for `%s'.", 
			   QueryInfo->UserName);
	return(DB_FALSE);
    case RS_NOMATCH:
	if (Debug) Message("Password was not found in history list.");
	return(DB_FALSE);
    case RS_MATCH:
	if (Debug) Message("Password was found in history list.");
	return(DB_TRUE);
    default:
	Error("%s: %s", Failed, ResultErrStr(&Result));
	return(DB_ERROR);
    }
}
