/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
/*
 * $Header: /src/common/cmd/spm/2.x/RCS/defs.h,v 1.70 1996/03/02 00:41:08 mcooper Exp $
 */

#ifndef __my_defs_h__
#define __my_defs_h__

#include "osconfig.h"
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <time.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/errno.h>
#include <sys/param.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <rpc/rpc.h>
#include <pwd.h>
#include "spm.h"
#include "config.h"
#include "version.h"

/*
 * Str*() Flags
 */
#define SF_RMLEADWS		0x01		/* Rm Leading whitespace */
#define SF_RMTRAILWS		0x02		/* Rm Trailing whitespace */
#define SF_RMWS			(SF_RMLEADWS|SF_RMTRAILWS)

/*
 * PDitem Flags
 *
 * Make sure PDitemFlagTable[] is kept in sync with these definetions.
 */
#define PDF_NOECHO		0x001		/* Don't echo input */
#define PDF_STRIPSEP		0x002		/* Strip seperators (-/) */

/*
 * Macros
 */
#define SV(s)			((s) ? s : "")
#define EQ(a,b)			(strcmp(a, b) == 0)
#define EQN(a,b,s)		(strncmp(a, b, s) == 0)
#define EQC(a,b)		(strcasecmp(a, b) == 0)
#define EQCN(a,b,s)		(strcasencmp(a, b, s) == 0)
#define IS_ON(v,f)		(v != 0 && (v & f))
#define FAIL(n,s)		if (ExitOnError) \
					exit(n); \
				else \
					return(s);

/*
 * Facility Names
 */
#define FN_FILES		"files"
#define FN_LOCAL		"local"
#define FN_NIS			"nis"
#define FN_NISPLUS		"nisplus"

/*
 * Key words
 */
#define KW_NONE			"none"
#define KW_YES			"Yes"
#define KW_NO			"No"
#define KW_NET			"net"
#define KW_HOST			"host"
		/* Age Keywords */
#define KW_MINCHG		"minchg"
#define KW_MAXVALID		"maxvalid"
#define KW_WARNTIME		"warntime"
#define KW_INACTIVE		"inactive"
#define KW_EXPIREDATE		"expiredate"
		/* PD Keywords */
#define KW_SSN			"ssn"
#define KW_DOB			"dob"
#define KW_MOMNAME		"momname"
#define KW_HOMEZIP		"homezip"

/*
 * Password Change String names
 */
#define PCS_PWD			"pwd"
#define PCS_SHELL		"shell"
#define PCS_GECOS		"gecos"

/*
 * Type definetions
 */
typedef void *			Opaque;		/* Hide data type */

/*
 * Generic Table
 */
typedef struct {
    char		       *Key;		/* The key */
    long			NumVal;		/* Numeric value */
} NameKey_t;

/*
 * Verify Status
 */
typedef enum {
    VS_ERROR = -1,				/* Verification error */
    VS_OKAY = 0,				/* Verified okay */
    VS_NOINFO,					/* No info available */
    VS_BADINFO					/* Info from user wrong */
} VERIFYstat_t;

/*
 * Password Change types
 */
typedef enum {
    PC_UNKNOWN = 0,				/* Unknown */
    PC_PWD,					/* Change Password */
    PC_GECOS,					/* Change GECOS */
    PC_SHELL					/* Change Shell */
} PCtype_t;

/*
 * Password facility types
 */
typedef enum {
    FAC_UNKNOWN,				/* Unknown facility */
    FAC_LOCAL,					/* Local /etc files */
    FAC_NIS,					/* NIS / Yellow Pages */
    FAC_NISPLUS					/* NIS+ */
} FACtype_t;

/*
 * Shadow Password information
 */
typedef struct {
    char		       *UserName;	/* UserName */
    char		       *CrPass;		/* Encrypted password */
} ShadPass_t;

/*
 * Password facility
 */
typedef struct {
    FACtype_t			Type;		/* Type of facility */
    char		       *Name;		/* Name of type */
    char		       *FullName;	/* Full name */
    char		       *PwdFile;	/* Name of pwd file */
    char		       *PwdLock;	/* Name of lock file */
    char		       *PostCmd;	/* Post op command */
    int			      (*Check)();	/* Function to check fac */
    SPMstat_t		      (*Change)();	/* Function to make changes */
    struct passwd	     *(*Getpwnam)();	/* Function to do getpwnam() */
    char		     *(*GetServer)();	/* Get name of server */
    char		     *(*GetDomain)();	/* Get domain name */
    char		       *ShadowFile;	/* Name of shadow pwd file */
    char		       *ShadowLock;	/* Name of shadow lock file */
    ShadPass_t		     *(*GetShadName)();	/* Get shadow pwd entry */
    char		     *(*UpdShadStr)();	/* Update shadow string */
} Facility_t;

/*
 * Database Table
 */
typedef struct {
    char		       *Type;		/* Type / Name */
    char		       *Desc;		/* Description */
    int			      (*Show)();	/* Show info function */
    int			      (*Read)();	/* Read function */
    int			      (*Write)();	/* Write function */
    int			      (*Set)();		/* Set value function */
    int			      (*Delete)();	/* Delete function */
} DBtable_t;

/*
 * Configuration Keys Table
 */
typedef struct {
    char		       *Key;		/* Keyword */
    char *		      (*Func)();	/* Function handler */
} CONFkeytab_t;

/*
 * Configuration Variables Table.
 */
typedef struct {
    char		       *Name;		/* Name of variable */
    enum {
	VT_STRING,
	VT_INT,
	VT_DATA
    }				Type;		/* Type of variable */
    Opaque		        Variable;	/* Variable to set */
    int			      (*Set)();		/* Set value function */
} CONFvartab_t;

/*
 * Client Information
 */
typedef struct {
    char		       *HostName;	/* What our host tables show */
    char		       *ClientName;	/* What client claims */
    char		       *AddrString;	/* Client address (string) */
    CLIENT		       *Client;		/* RPC CLIENT info */
} ClientInfo_t;

/*
 * Database function return values
 */
enum DBstatus {
    DB_ERROR = -1,				/* Error occured */
    DB_OK = 0,					/* Success */
    DB_NOENT = 1,				/* No {such,more} entry */
    DB_TRUE = 2,				/* True */
    DB_FALSE = 3				/* False */
};
typedef enum DBstatus		DBstatus_t;

/*
 * Database info
 */
typedef struct {
    char		       *FileName;	/* Name of db file */
    int				Flags;		/* Flags for db */
    Opaque			Info;		/* DB specific data */
} DBinfo_t;

/*
 * Database argument type
 */
typedef struct {
    void		       *Data;		/* Data value */
    size_t			Size;		/* Size of Data */
} DBarg_t;

/*
 * Password lock information
 */
typedef struct {
    int				FileDesc;	/* Lock file descriptor */
    FILE		       *FilePtr;	/* Lock file pointer */
    char		       *LockFile;	/* Lock file name */
    char		       *Error;		/* Error string */
} PWDlock_t;

/*
 * Gecos fields
 */
struct _Gecos {
    char		       *Label;		/* Label/name */
    char		       *Example;	/* Example of format */
    int				MaxSize;	/* Max field size */
    /* These are only used internally */
    char		       *OldVal;		/* Old value */
    char		       *NewVal;		/* New value */
    struct _Gecos	       *Next;		/* Next element */
};
typedef struct _Gecos		Gecos_t;

/*
 * Verification types
 */
#define V_SSN			"SSN"		/* Social Security Number */
#define V_DOB			"DOB"		/* Date Of Birth */
#define V_MOMNAME		"MomName"	/* Mom's maiden name */
#define V_HOMEZIP		"HomeZip"	/* User's Home ZIP code */

/*
 * Verification information
 */
typedef struct {
    char		       *Name;		/* Name of verification */
    char		       *Desc;		/* Description of info */
    VERIFYstat_t	      (*AskUser)();	/* Function to ask user */
} Verify_t;

/*
 * Password history
 */
typedef struct {
    string_t			UserName;	/* User name */
    string_t			List;		/* List of passwords */
    SPMresult_t			Result;
} PwdHist_t;

/*
 * Key/Value 
 */
struct _KeyVal {
    char		       *Key;		/* Key string */
    char		       *Value;		/* Value string */
    struct _KeyVal	       *Next;		/* Next entry */
};
typedef struct _KeyVal 		KeyVal_t;

/*
 * Access information
 */
typedef struct {
    char		       *Name;		/* Name of list */
    KeyVal_t		       *List;		/* List of accesses */
} AccessInfo_t;

/* 
 * Flags for Transact_t->Flags
 */
#define TF_IGNORE		0x01		/* Ignore entry */
#define TF_TODO			0x02		/* Mark it TODO */
#define TF_DONE			0x04		/* Mark completed */

/*
 * Transaction
 */
struct _Transact {
    /* Members used for processing */
    char		       *Ident;		/* Unique ID */
    char		       *File;		/* Name of file */
    int				Desc;		/* File Descriptor */
    char		       *Domain;		/* Domain name for change */
    int				Flags;		/* Status flags */
    SPMresult_t			Result;		/* Results of processing */
    /* Actual Data fields */
    char		       *UserName;	/* UserName */
    PCtype_t			Type;		/* Type of change */
    char		       *OldValue;	/* Old value */
    char		       *NewValue;	/* New value */
    struct _Transact	       *Discards;	/* List of discard items */
    struct _Transact	       *Next;		/* Next member of list */
    struct _Transact	       *Prev;		/* Previous member of list */
};
typedef struct _Transact	Transact_t;

/*
 * Declarations
 */
extern AccessInfo_t		PDaccess;
extern AccessInfo_t		SPMaccess;
extern CONFvartab_t	        VarTable[];
extern char		       *AgeDir;
extern char		       *GecosSep;
extern Gecos_t		       *GecosList;
extern PDitem_t		       *PDitemList;
extern char		       *DomainName;
extern char		       *LocalPostCmd;
extern char		       *LocalPwdFile;
extern char		       *LocalPwdLock;
extern char		       *LocalShadowFile;
extern char		       *LocalShadowLock;
extern char		       *NISPostCmd;
extern char		       *NISPwdFile;
extern char		       *NISPwdLock;
extern char		       *NISdir;
extern char		       *PDdir;
extern char		       *PDserver;
extern char		       *PWHdir;
extern char		       *ProgramName;
extern char		       *SPMserver;
extern char		       *StripSepChars;
extern char		       *TransFacList;
extern char		       *TransDir;
extern char		       *VerifyList;
extern int			AmPrived;
extern int			AskTries;
extern int			BackGround;
extern int			Debug;
extern int			DoKeys;
extern int			ExitOnError;
extern int			LockRetryInt;
extern int			MaxAsk;
extern int			MaxLockTries;
extern int			MaxPWHent;
extern int			MaxGecosSize;
extern int			MaxPwdSize;
extern int			MaxShellSize;
extern int			UseSyslog;
extern int			Verbose;
extern int			VerifyNum;

PCtype_t			GetPCtype();
VERIFYstat_t			VERIFYuser();
char			       *Basename();
char			       *CheckDomain();
char			       *GetDomain();
char			       *GetHostName();
char			       *GetKey();
char			       *GetPCname();
char			       *GetSPMserver();
char			       *GetValue();
char			       *GetVersion();
char			       *ResultErrStr();
char			       *TrimWhite();
char			       *getcrypt();
char			       *itoa();
char			       *ltoa();
char			       *ultoa();
extern ClientInfo_t	       *AUTHcheck();
extern char		       *GetTimeStr();
extern int			GetResSock();
extern void		       *xcalloc();
extern void		       *xmalloc();
int				ACCESSset();
time_t				TIMEparse();
void				QueryInit();
void				ResultInit();
int				TTYechoOff();
int				TTYechoOn();
#if	defined(__STDC__)
void				Error(char *Fmt, ...);
char			       *Format(char *Fmt, ...);
void				Message(char *Fmt, ...);
#else	/* !__STDC__ */
void				Error();
char 			       *Format();
void				Message();
#endif	/* __STDC__ */

/*
 * OS functions
 */
char			       *crypt();
char			       *getpass();
char			       *getusershell();
char			       *mktemp();
char 			       *strerror();
struct passwd		       *fgetpwent();

extern int			errno;

/*
 * Configuration calls
 */
void				CONFinstall();
void				CONFinit();
int				CONFparse();

/*
 * Database calls
 */
DBinfo_t		       *DBopen();
DBstatus_t			DBcheckpwd();
DBstatus_t			DBclose();
DBstatus_t			DBdelete();
DBstatus_t			DBget();
DBstatus_t			DBiterate();
DBstatus_t			DBstore();
DBtable_t		       *DBgettable();
char			       *DBgetkeyuser();
void				DBlist();

/*
 * Password calls
 */
PWDlock_t		       *PWDlock();
SPMstat_t			PWDchange();
char			       *PWDgettmp();
char			       *PWDpwdtostr();
char			       *PWDvalidfield();
int				PWDmatch();
void				PWDunlock();
int				PWDvalidlock();
int				PWDverify();
struct passwd		       *PWDstrtopwd();
void				PWDpost();
Gecos_t			       *PWDgetgecos();
ShadPass_t		       *SPWDgetShadPass();
char			       *SPWDupdSpwdStr();


/*
 * Age Calls
 */
AGEinfo_t		       *_AGEgetbyuser();
AGEinfo_t		       *AGEgetbyuser();
AGEinfo_t		       *AGEgetinfo();
char			       *AGEadd();
int			        AGEdelete();
char			       *AGEgetfile();
char			       *AGEgetstr();
char			       *AGEupdate();
int				AGEread();
int				AGEshow();
int				AGEwrite();
int				AGEset();
void				AGEinit();

/*
 * PD Calls
 */
PDinfo2_t		       *_PDgetbyuser();
PDinfo2_t		       *PDgetbyuser();
PDitem_t		       *PDgetItem();
char			       *PDgetfile();
char			       *PDgetstr();
char			       *PDwhatSet();
PDinfo2_t		       *PDgetinfo();
PDinfo2_t		       *PDinfo1to2();
PDinfo_t		       *PDinfo2to1();
int				PDshow();
int				PDread();
int				PDwrite();
int				PDset();
int				PDdelete();
long				PDgetItemFlag();
void				PDinit();
void				PDitemAddList();

/*
 * Password History Calls
 */
PwdHist_t		       *PWHgetbyuser();
PwdHist_t		       *PWHgetinfo();
char			       *PWHgetfile();
int				PWHadd();
int				PWHdelete();
int				PWHinsert();
int				PWHread();
int				PWHset();
int				PWHshow();
int				PWHwrite();
void				PWHinit();

/*
 * Client Calls
 */
ClientInfo_t		       *CLNTconnect();
ClientInfo_t		       *CLNTcreate();
int				CLNTcall();
void				CLNTdestroy();
char			       *CLNTerrstr();
void				CLNTinit();

/*
 * Facilities
 */
extern Facility_t		Facilities[];
Facility_t		       *FACget();
Facility_t		       *FACgetfac();
Facility_t		       *FAClookup();
char			       *FACgetByType();
SPMstat_t			FACchange();

/*
 * Files
 */
int				LOCALcheck();
char			       *LOCALgetDomain();
char			       *LOCALgetserver();
struct passwd		       *LOCALgetpwnam();
ShadPass_t		       *LOCALgetShadPass();

/*
 * NIS
 */
int				NIScheck();
char			       *NISgetDomain();
char			       *NISgetserver();
struct passwd		       *NISgetpwnam();

/*
 * NIS+
 */
int				NISPLUScheck();
SPMstat_t			NISPLUSchange();
char			       *NISPLUSgetDomain();
char			       *NISPLUSgetserver();
struct passwd		       *NISPLUSgetpwnam();

#if	defined(HAVE_USERPW)
/*
 * userpw
 */
SPMstat_t			USERPWchange();
ShadPass_t		       *USERPWgetShadPass();
#endif	/* HAVE_USERPW */

#if	defined(HAVE_HPUXPW)
/*
 * HPUX password routines
 */
ShadPass_t		       *HPUXPWgetShadPass();
char			       *HPUXPWupdShadStr();
#endif	/* HAVE_HPUXPW */

/*
 * Ask functions
 */
int				ASKpassword();
int			        ASKprompt();
char			       *ASKnewpwd();
char			       *ASKgecos();
char			       *ASKshell();

/*
 * Transaction functions
 */
SPMstat_t		        TRANSadd();
Transact_t		       *TRANSgetByUser();
Transact_t		       *TRANSgetTrans();
Transact_t		       *TRANSread();
char			       *TRANSgetDir();
char			       *TRANSgetIdent();
int				TRANSdelete();
int				TRANSenabled();
int				TRANScheck();
void				TRANSinsert();

/*
 * String Utility functions
 */
int				StrToArgv();
int				StrIsSep();
char			       *StrNext();
char			       *StrStripSep();
void				DestroyArgv();

#endif	/* __my_defs_h__ */
