/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/pd.c,v 1.28 1996/03/02 00:42:12 mcooper Exp $";
#endif	/* lint */

/*
 * Personel Data functions
 */

#include "defs.h"

/*
 * The old, hardwired PD field names (in order)
 */
static char	       *OldPDkeys[] = { KW_SSN, KW_DOB, 
					KW_MOMNAME, KW_HOMEZIP };
#define NumPDkeys	(sizeof(OldPDkeys)/sizeof(char *))

/*
 * Show all items in Item list
 */
void
PDshowItem(Item)
    PDitem_t		       *Item;
{
    register PDitem_t	       *Ptr;
    register PDitem_t	       *ConfPDitem;

#define AS(s)	( (s) ? s : "(unknown)" )
#define AN(s)	( (s) ? strlen(s) : 0 )
    for (Ptr = Item; Ptr; Ptr = Ptr->Next) {
	ConfPDitem = PDgetItem(Ptr->Key, PDitemList);
	if (ConfPDitem)
	    printf("%s (%s)%*s: ", AS(ConfPDitem->Desc), AS(ConfPDitem->Key),
		   37 - (AN(ConfPDitem->Desc) + strlen(ConfPDitem->Key)), "");
	else
	    printf("(%s)%*s: ", AS(Ptr->Key), 38 - strlen(Ptr->Key), "");
	printf("%s\n", AS(Ptr->Value));
    }
#undef AN
}

/*
 * Show personel data information about a given User.
 */
int 
PDshow(UserName, Info)
    char		       *UserName;
    PDinfo2_t		       *Info;
{
    if (!Info->UserName) {
	Error("No personel data available for \"%s\".", UserName);
	return(-1);
    }

    printf("%-40s: %s\n", "User Name", UserName);
    printf("%-40s: %s\n", "Entry Created", AS(GetTimeStr(Info->CrtTime)));
    printf("%-40s: %s\n", "Entry Last Modified", 
	   AS(GetTimeStr(Info->ModTime)));
#undef AS

    PDshowItem(Info->Items);

    return(0);
}

/*
 * Initialize a PDinfo2_t.
 * All strings must be non-NULL for XDR.
 */
void
PDinfo2Init(InfoPtr)
    PDinfo2_t		       *InfoPtr;
{
    (void) memset((char *)InfoPtr, 0, sizeof(PDinfo2_t));
    InfoPtr->UserName = "";
    InfoPtr->ReqUser = "";
    InfoPtr->Domain = "";
    InfoPtr->CurPass = "";
    InfoPtr->FacName = "";
    ResultInit(&InfoPtr->Result);
}

/*
 * Free a PDitem linked list
 */
void
PDfreeItemList(List)
    PDitem_t		       *List;
{
    register PDitem_t	       *Ptr;
    register PDitem_t	       *Save;

    for (Ptr = List; Ptr; ) {
	if (Ptr->Key)		(void) free(Ptr->Key);
	if (Ptr->Value)		(void) free(Ptr->Value);
	if (Ptr->Desc)		(void) free(Ptr->Desc);
	if (Ptr->Example)	(void) free(Ptr->Example);
	Save = Ptr->Next;
	(void) free(Ptr);
	Ptr = Save;
    }
}

/*
 * Get a PDitem
 */
PDitem_t *
PDgetItem(Key, ItemList)
    char		       *Key;
    PDitem_t		       *ItemList;
{
    register PDitem_t	       *Ptr;

    for (Ptr = ItemList; Ptr; Ptr = Ptr->Next)
	if (EQC(Key, Ptr->Key))
	    return(Ptr);

    return((PDitem_t *) NULL);
}

/*
 * Get key from PD string.
 */
char *
PDgetkey(String)
    char		       *String;
{
    return(DBgetkeyuser(String));
}

/*
 * Get name of PD database file
 */
char *
PDgetfile(DomainName, ErrBuff)
    char		       *DomainName;
    char		       *ErrBuff;
{
    static char		        File[MAXPATHLEN];
    static char		        Buff[BUFSIZ];
    char		       *Domain;
    char		       *Err = NULL;

    if (ErrBuff)
	Err = ErrBuff;
    else
	Err = Buff;

    Domain = (DomainName) ? DomainName : GetDomain(NULL);

    if (strlen(PDdir) + strlen(Domain) + 2 >= sizeof(File)) {
	(void) sprintf(Err, "Size of \"%s\" + \"%.100s\" is too large.", 
		       PDdir, Domain);
	if (!ErrBuff)
	    Error("%s", Err);
	return((char *)NULL);
    }

    (void) sprintf(File, "%s/%s", PDdir, Domain);

    return(File);
}

/*
 * Convert an PDinfo2_t data type to a string.
 */
char *
PDgetstr(Info)
    PDinfo2_t		       *Info;
{
    static char			Buff[MAXDBVAL];
    register int		Len = 0;
    register PDitem_t	       *Item;

    if (!Info)
	return((char *) NULL);

    (void) sprintf(Buff, "%s:%d:%d",
		   Info->UserName,
		   Info->CrtTime,
		   Info->ModTime);

    for (Item = Info->Items; Item && Item->Key;
	 Item = Item->Next) {
	if (!Item->Key || !Item->Value)
	    continue;
	Len = strlen(Buff);
	if ((Len + strlen(Item->Key) + strlen(Item->Value) + 3) >= 
	    sizeof(Buff)) {
	    Error("PDgetstr: DB Buffer too small (%d)", sizeof(Buff));
	    break;
	}

	if (Item->Value)
	    sprintf(&Buff[Len], ":%s=%s", Item->Key, Item->Value);
    }

    return(Buff);
}

/*
 * Add New to end of link list pointed at by ListPtr.
 */
void
PDitemAddList(ListPtr, New)
    PDitem_t		      **ListPtr;
    PDitem_t		       *New;
{
    register PDitem_t	       *Ptr;

    if (*ListPtr) {
	for (Ptr = *ListPtr; Ptr && Ptr->Next; Ptr = Ptr->Next);
	Ptr->Next = New;
    } else
	*ListPtr = New;
}

/*
 * Parse a db string of PDitem's into a linked list of PDitem.
 */
PDitem_t *
PDparseItems(String)
    char		       *String;
{
    PDitem_t		       *ItemList = NULL;
    PDitem_t		       *Item;
    register PDitem_t	       *New;
    register int		i;
    register char	       *Str;
    register char	       *cp;

    for (Str = strtok(String, ":"), i = 0; Str; Str = strtok(NULL, ":"), ++i) {
	New = NULL;
	if (cp = strchr(Str, '=')) {
	    /*
	     * This is one of the new "key=value" entries.
	     */
	    New = (PDitem_t *) xcalloc(1, sizeof(PDitem_t));
	    *cp = CNULL;
	    New->Key = strdup(Str);
	    New->Value = strdup(cp + 1);
	    *cp = '=';
	} else if (i < NumPDkeys) {
	    /*
	     * This must be one of the old PD keys.
	     */
	    New = (PDitem_t *) xcalloc(1, sizeof(PDitem_t));
	    New->Key = OldPDkeys[i];
	    New->Value = Str;
	}
	if (New && New->Key) {
	    /*
	     * Fill in Desc and Example fields from values found in spm.conf
	     */
	    if (Item = PDgetItem(New->Key, PDitemList)) {
#define SET(a, b)	if (a && *a) b = strdup(a)
		SET(Item->Desc, New->Desc);
		SET(Item->Example, New->Example);
		New->Flags = Item->Flags;
#undef SET
	    }
	}
	if (New)
	    PDitemAddList(&ItemList, New);
    }

    return(ItemList);
}

/*
 * Convert a string into an PDinfo2_t data type.
 */
PDinfo2_t *
PDgetinfo(String)
    char		       *String;
{
    static PDinfo2_t	       *Info = NULL;
    register int		Len;
    char	       	       *cp;
    char		       *End;

    if (!Info)
	Info = (PDinfo2_t *) xcalloc(1, sizeof(PDinfo2_t));
    else {
	if (Info->UserName)	(void) free(Info->UserName);
	if (Info->Items)	PDfreeItemList(Info->Items);
	Info->Items = NULL;
    }
    memset((char *)Info, 0, sizeof(PDinfo2_t));	/* Zap Info */

    /* XXX - String is harmed by strtok() */
    End = &String[strlen(String)];
    if (cp = strtok(String, ":"))
	Info->UserName = strdup(cp);
    else
	return((PDinfo2_t *) NULL);

    if (cp = strtok(NULL, ":"))
	Info->CrtTime = strtol(cp, NULL, 0);
    else
	return((PDinfo2_t *) NULL);

    if (cp = strtok(NULL, ":"))
	Info->ModTime = strtol(cp, NULL, 0);
    else
	return((PDinfo2_t *) NULL);

    if (cp = strtok(NULL, ":")) {
	/*
	 * Undo the CNULL in String (cp) that strtok() inserted so that
	 * we have the rest of the String buffer to pass on.
	 */
	Len = strlen(cp);
	if (&cp[Len] < End)
	    *(&cp[Len]) = ':';
	Info->Items = PDparseItems(cp);
    }

    return(Info);
}

/*
 * Get pd entry by matching UserName.
 */
PDinfo2_t *
PDgetbyuserdb(DBinfo, UserName)
    DBinfo_t		       *DBinfo;
    char		       *UserName;
{
    static PDinfo2_t	        Info;
    static DBarg_t	        DBkey;
    static DBarg_t	        DBresult;
    PDinfo2_t		       *PDptr;
    char		       *StrPtr;
    int				Status;

    errno = 0;
    if (!DBinfo)
	return((PDinfo2_t *) NULL);

    DBkey.Data = (void *) UserName;
    DBkey.Size = (size_t) strlen(UserName);
    DBresult.Data = (void *) NULL;
    DBresult.Size = (size_t) 0;

    PDinfo2Init(&Info);

    Status = DBget(DBinfo, &DBkey, &DBresult);

    if (Status == 0) {			/* Success */
	StrPtr = (char *) DBresult.Data;
	StrPtr[DBresult.Size] = CNULL;
	if (PDptr = PDgetinfo(StrPtr)) {
	    PDptr->Result.Status = RS_OK;
	    PDptr->Result.Message = "";	/* Required for XDR */
	    return(PDptr);
	}
    } else if (Status == 1) {		/* No such entry */
	Info.Result.Status = RS_NOENT;
    } else {				/* Error occured */
	Info.Result.Message = SYSERR;
    }

    return(&Info);
}

#if	defined(PD_SSN_FILE)
/*
 * Look up the SSN for UserName in a special, SSN-only, DB.
 * The DB has a data record format of:
 *
 *		username:SSN
 */
char *
PDgetSSN(UserName)
    char		       *UserName;
{
    char		       *File;
    DBinfo_t		       *DBinfo;
    static char		        Buff[BUFSIZ];
    static DBarg_t	        DBkey;
    static DBarg_t	        DBresult;
    char		       *StrPtr;
    int				Status;
    register char	       *cp;

    File = PDgetfile(PD_SSN_FILE, Buff);
    if (!File)
	return((char *) NULL);

    if (!(DBinfo = DBopen(File, O_RDONLY)))
	return((char *) NULL);
	
    DBkey.Data = (void *) UserName;
    DBkey.Size = (size_t) strlen(UserName);
    DBresult.Data = (void *) NULL;
    DBresult.Size = (size_t) 0;

    Status = DBget(DBinfo, &DBkey, &DBresult);

    (void) DBclose(DBinfo);

    if (Status != 0)
	return((char *) NULL);

    StrPtr = (char *) DBresult.Data;
    StrPtr[DBresult.Size] = CNULL;
    if (cp = strchr(StrPtr, ':'))
	++cp;
    else
	cp = StrPtr;

    return(cp);
}
#endif	/* PD_SSN_FILE */

/*
 * Get pd entry by matching UserName.
 */
PDinfo2_t *
_PDgetbyuser(UserName, Domain)
    char		       *UserName;
    char		       *Domain;
{
    char		       *File;
    static PDinfo2_t		Info;
    PDinfo2_t		       *InfoPtr;
    DBinfo_t		       *DBinfo;
    static char		        Buff[BUFSIZ];

    /*
     * Prepare error result in case we need it.
     */
    PDinfo2Init(&Info);

    File = PDgetfile(Domain, Buff);
    if (!File) {
	Error("%s", Info.Result.Message = Buff);
	return(&Info);
    }

    if (!(DBinfo = DBopen(File, O_RDONLY))) {
	if (errno == ENOENT)
	    Info.Result.Status = RS_NOENT;
	(void) sprintf(Buff, "Cannot open %s: %s.", File, SYSERR);
	Error("%s", Info.Result.Message = Buff);
	return(&Info);
    }
	
    InfoPtr = PDgetbyuserdb(DBinfo, UserName);

    (void) DBclose(DBinfo);

    return (InfoPtr);
}

/*
 * Get pd entry by matching UserName.
 */
PDinfo2_t *
PDgetbyuser(UserName, Domain)
    char		       *UserName;
    char		       *Domain;
{
    static PDinfo2_t		Info;
    static PDitem_t		PDitemLoc;
    register PDitem_t	       *Item;
    PDinfo2_t		       *InfoPtr;
    char		       *SSN = NULL;

    InfoPtr = _PDgetbyuser(UserName, Domain);
#if	defined(PD_SSN_FILE)
    if (!InfoPtr || InfoPtr->Result.Status != RS_OK) {
	if (Debug) 
	    Message("No PD entry for `%s' in std `%s' db.",
		    UserName, Domain);
	SSN = PDgetSSN(UserName);
	if (Debug && SSN)
	    Message("Found SSN db entry for `%s': `%s'", UserName, SSN);
	else if (Debug && !SSN)
	    Message("No SSN db entry for `%s'", UserName);
	if (SSN) {
	    /*
	     * Handle result from special SSN only database.
	     */
	    /*
	     * See if we can find an "SSN" entry in the PDitemList
	     */
	    Item = PDgetItem(KW_SSN, PDitemList);
	    if (!Item) {
		/*
		 * We didn't find the "SSN" global PDitem so we create
		 * our own.
		 */
		(void) memset((char *)&PDitemLoc, CNULL, sizeof(PDitemLoc));
		PDitemLoc.Key = KW_SSN;
		PDitemLoc.Desc = "Social Security Number";
		PDitemLoc.Example = "123-45-6789";
		Item = &PDitemLoc;
	    }
	    Item->Value = SSN;
	    PDinfo2Init(&Info);
	    Info.UserName = UserName;
	    Info.Items = Item;
	    Info.Result.Status = RS_OK;
	    InfoPtr = &Info;
	}
    }
#endif	/* PD_SSN_FILE */

    return(InfoPtr);
}

/*
 * Read PD information.
 */
int
PDread(FileName, DomainName)
    char		       *FileName;
    char		       *DomainName;
{
    char		       *DBfile;
    DBinfo_t		       *DBinfo;
    static DBarg_t		DBkey;
    static DBarg_t		DBdata;
    int				Status;
    FILE		       *fp;

    if (!FileName || EQ(FileName, "-"))
	fp = stdout;
    else
	if (!(fp = fopen(FileName, "w+"))) {
	    Error("Cannot open \"%s\" for writing: %s.", FileName, SYSERR);
	    exit(1);
	}

    DBfile = PDgetfile(DomainName, NULL);
    if (!DBfile) {
	Error("Cannot determine database file name.");
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    if (!(DBinfo = DBopen(DBfile, O_RDONLY))) {
	Error("Cannot open \"%s\" for reading: %s.", DBfile, SYSERR);
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    for (Status = DBiterate(DBinfo, &DBkey, &DBdata, TRUE); Status == 0; 
	 Status = DBiterate(DBinfo, &DBkey, &DBdata, FALSE)) {
	if (DoKeys)
	    (void) fprintf(fp, "%.*s ", DBkey.Size, DBkey.Data);
	(void) fprintf(fp, "%.*s\n", DBdata.Size, DBdata.Data);
    }

    if (Status == -1)
	Error("Database sequential fetch error: %s.", SYSERR);
    else if (Status == 1)	/* Normal "end of entries" indicator */
	Status = 0;

    (void) DBclose(DBinfo);
    if (fp != stdin)
	(void) fclose(fp);

    return(Status);
}

/*
 * Write PD information
 */
int
PDwrite(FileName, DomainName)
    char		       *FileName;
    char		       *DomainName;
{
    char		       *DBfile;
    static char			Buff[MAXDBVAL];
    DBinfo_t		       *DBinfo;
    static DBarg_t		DBkey;
    static DBarg_t		DBdata;
    register int		LineNo;
    register int		Len;
    int				Status = -1;
    FILE		       *fp;

    if (!FileName || EQ(FileName, "-"))
	fp = stdin;
    else
	if (!(fp = fopen(FileName, "r"))) {
	    Error("Cannot open \"%s\" for reading: %s.", FileName, SYSERR);
	    exit(1);
	}

    DBfile = PDgetfile(DomainName, NULL);
    if (!DBfile) {
	Error("Cannot determine database file name.");
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    if (!(DBinfo = DBopen(DBfile, O_RDWR|O_CREAT))) {
	Error("Cannot open \"%s\" for writing: %s.", DBfile, SYSERR);
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    LineNo = 0;
    while (fgets(Buff, sizeof(Buff), fp)) {
	++LineNo;
	Len = strlen(Buff);
	if (Buff[Len - 1] == '\n')
	    Buff[--Len] = CNULL;

	if (!(DBkey.Data = (void *) PDgetkey(Buff))) {
	    Error("Error: line %d: Cannot parse key.", LineNo);
	    Status = 1;
	    continue;
	}
	DBkey.Size = (size_t) strlen(DBkey.Data);
	DBdata.Data = (void *) Buff;
	DBdata.Size = (size_t) Len;

	if ( DBstore(DBinfo, &DBkey, &DBdata, 0) != 0) {
	    Error("Error: line %d: Database write failed: %s.", 
		  LineNo, SYSERR);
	    Status = 1;
	}
    }

    (void) DBclose(DBinfo);
    if (fp != stdin)
	(void) fclose(fp);

    return(Status);
}

static void 
PDwhatCheck(Buffer, String, Label)
    char		       *Buffer;
    char		       *String;
    char		       *Label;
{
    if (String && *String) {
	if (Buffer[0])
	    (void) strcat(Buffer, ",");
	(void) strcat(Buffer, Label);
    }
}

/*
 * Return a string list of what queryable fields are set
 */
char *
PDwhatSet(Info)
    PDinfo2_t		       *Info;
{
    static char			Buff[BUFSIZ];
    register PDitem_t	       *Item;

    Buff[0] = CNULL;
    for (Item = Info->Items; Item; Item = Item->Next)
	PDwhatCheck(Buff, Item->Value, Item->Key);

    return( (Buff[0]) ? Buff : (char *) NULL );
}

/*
 * Add pd entry to database.
 */
char *
PDadd(UserName, Domain, Info)
    char		       *UserName;
    char		       *Domain;
    PDinfo2_t		       *Info;
{
    static char			ErrBuff[BUFSIZ];
    static DBarg_t		DBkey;
    static DBarg_t		DBdata;
    char		       *DBfile;
    DBinfo_t		       *DBinfo;
    char		       *Status = NULL;

    ErrBuff[0] = CNULL;
    DBfile = PDgetfile(Domain, ErrBuff);
    if (!DBfile)
	return(ErrBuff);

    if (!(DBinfo = DBopen(DBfile, O_RDWR|O_CREAT)))
	return(Format("Cannot open \"%s\" for writing: %s.", DBfile, SYSERR));

    DBkey.Data = (void *) UserName;
    DBkey.Size = (size_t) strlen(DBkey.Data);
    DBdata.Data = (void *) PDgetstr(Info);
    DBdata.Size = (size_t) strlen(DBdata.Data);

    if (DBstore(DBinfo, &DBkey, &DBdata, 0) != 0)
	Status = Format("Database write failed: %s", SYSERR);

    (void) DBclose(DBinfo);

    return(Status);
}

/*
 * Set an Pd argument.
 *
 * SetArg looks like "key1=value1,key2=value2,....".
 */
int
PDset(UserName, Domain, SetArg)
    char		       *UserName;
    char		       *Domain;
    char		       *SetArg;
{
    PDinfo2_t		       *Info;
    static PDitem_t		Item;
    PDitem_t		       *ItemPtr;
    static char			ErrBuff[BUFSIZ];
    char		       *ErrStr = NULL;
    char		       *Entry;
    char		       *Key;
    char		       *Value;

    Info = PDgetbyuser(UserName, Domain);
    if (!Info || Info->Result.Status != RS_OK) {
	if (Debug) Message(
	   "WARNING: No current pd entry for %s@%s: Creating new entry.",
			   UserName, Domain);
	Info = (PDinfo2_t *) xmalloc(sizeof(PDinfo2_t));
	PDinfo2Init(Info);
	Info->UserName = UserName;
    }
    /*
     * Set creation time if not already set.
     */
    if (Info->CrtTime == 0)
	time(&(Info->CrtTime));

    for (Entry = strtok(SetArg, ","); Entry; Entry = strtok(NULL, ",")) {
	Key = GetKey(Entry, ErrBuff);
	if (!Key) {
	    Error("Error parsing key from `%s': %s.", Entry, ErrBuff);
	    exit(1);
	}
	Value = GetValue(Entry, ErrBuff);

	if (ItemPtr = PDgetItem(Key, Info->Items)) {
	    /*
	     * Update existing entry
	     */
	    ItemPtr->Key = Key;
	    ItemPtr->Value = Value;
	} else {
	    /*
	     * Create new entry
	     */
	    (void) memset((char *)&Item, CNULL, sizeof(PDitem_t));
	    Item.Key = Key;
	    Item.Value = Value;
	    PDitemAddList(&(Info->Items), &Item);
	}
    }

    (void) time(&(Info->ModTime));
    ErrStr = PDadd(UserName, Domain, Info);
    if (ErrStr) {
	Error("Update of pd entry for %s@%s failed: %s.",
	      UserName, Domain, ErrStr);
	return(-1);
    }

    /* Success! */
    return(0);
}

/*
 * Delete pd entry from database.
 */
int
PDdelete(UserName, Domain)
    char		       *Domain;
    char		       *UserName;
{
    static char			ErrBuff[BUFSIZ];
    static DBarg_t		DBkey;
    char		       *DBfile;
    DBinfo_t		       *DBinfo;
    int				Status = 0;

    ErrBuff[0] = CNULL;
    DBfile = PDgetfile(Domain, ErrBuff);
    if (!DBfile) {
	Error("%s", ErrBuff);
	return(-1);
    }

    if (!(DBinfo = DBopen(DBfile, O_RDWR|O_CREAT))) {
	Error("Cannot open \"%s\" for writing: %s.", DBfile, SYSERR);
	return(-1);
    }

    DBkey.Data = (void *) UserName;
    DBkey.Size = (size_t) strlen(UserName);

    if (DBdelete(DBinfo, &DBkey) != 0) {
	Error("Database delete failed: %s", SYSERR);
	Status = -1;
    }

    (void) DBclose(DBinfo);

    return(Status);
}

/*
 * Lookup string Flag in the PDitem Table
 */
long
PDgetItemFlag(Flag)
    char		       *Flag;
{
    extern NameKey_t		PDitemFlagTable[];
    register int		i;

    for (i = 0; PDitemFlagTable[i].Key; ++i)
	if (EQC(Flag, PDitemFlagTable[i].Key))
	    return(PDitemFlagTable[i].NumVal);

    return(0);
}

/*
 * Convert a PDinfo2_t to PDinfo_t
 */
PDinfo_t *
PDinfo2to1(Info)
    PDinfo2_t		       *Info;
{
    static PDinfo_t		New;
    PDitem_t		       *Item;

    New.UserName = SV(Info->UserName);
    New.ReqUser = SV(Info->ReqUser);
    New.Domain = SV(Info->Domain);
    New.CurPass = SV(Info->CurPass);
    New.FacName = SV(Info->FacName);
    New.CrtTime = Info->CrtTime;
    New.ModTime = Info->ModTime;
    (void) memcpy((char *)&(New.Result), (char *)&(Info->Result), 
		  sizeof(SPMresult_t));
    for (Item = Info->Items; Item; Item = Item->Next) {
	if (EQC(Item->Key, KW_SSN))
	    New.SSN = SV(Item->Value);
	else if (EQC(Item->Key, KW_DOB))
	    New.DOB = SV(Item->Value);
	else if (EQC(Item->Key, KW_MOMNAME))
	    New.MomName = SV(Item->Value);
	else if (EQC(Item->Key, KW_HOMEZIP))
	    New.HomeZip = SV(Item->Value);
    }

    return(&New);
}

/*
 * Convert a PDinfo_t to PDinfo2_t
 */
PDinfo2_t *
PDinfo1to2(Info)
    PDinfo_t		       *Info;
{
    static PDinfo2_t		New;
    static PDitem_t	       *ItemList;
    register PDitem_t	       *Item;
    register PDitem_t	       *ItemPtr;

    if (ItemList) {
	/*
	 * Be paranoid and insure Value is set to "" for XDR.
	 */
	for (ItemPtr = ItemList; ItemPtr; ItemPtr = ItemPtr->Next)
	    ItemPtr->Value = "";
    } else {
	/*
	 * The first time we're called we allocate memory and build
	 * ItemList.  This list is reused on subsequent calls.
	 */
	Item = (PDitem_t *) xcalloc(1, sizeof(PDitem_t));
	Item->Key = KW_SSN;
	PDitemAddList(&ItemList, Item);

	Item = (PDitem_t *) xcalloc(1, sizeof(PDitem_t));
	Item->Key = KW_DOB;
	PDitemAddList(&ItemList, Item);

	Item = (PDitem_t *) xcalloc(1, sizeof(PDitem_t));
	Item->Key = KW_MOMNAME;
	PDitemAddList(&ItemList, Item);

	Item = (PDitem_t *) xcalloc(1, sizeof(PDitem_t));
	Item->Key = KW_HOMEZIP;
	PDitemAddList(&ItemList, Item);
    }

    if (Item = PDgetItem(KW_SSN, ItemList))
	Item->Value = SV(Info->SSN);
    if (Item = PDgetItem(KW_DOB, ItemList))
	Item->Value = SV(Info->DOB);
    if (Item = PDgetItem(KW_MOMNAME, ItemList))
	Item->Value = SV(Info->MomName);
    if (Item = PDgetItem(KW_HOMEZIP, ItemList))
	Item->Value = SV(Info->HomeZip);

    New.UserName = SV(Info->UserName);
    New.ReqUser = SV(Info->ReqUser);
    New.Domain = SV(Info->Domain);
    New.CurPass = SV(Info->CurPass);
    New.FacName = SV(Info->FacName);
    New.CrtTime = Info->CrtTime;
    New.ModTime = Info->ModTime;
    New.Items = ItemList;
    (void) memcpy((char *)&(New.Result), (char *)&(Info->Result), 
		  sizeof(SPMresult_t));

    return(&New);
}
