/*
 * Copyright 1995, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/pwh.c,v 1.12 1996/02/08 00:44:17 mcooper Exp $";
#endif	/* lint */

/*
 * Password History functions
 */

#include "defs.h"

/*
 * Initialize Ptr.
 */
void
PWHinit(Ptr)
    PwdHist_t		       *Ptr;
{
    memset(Ptr, 0, sizeof(PwdHist_t));
    Ptr->UserName = "";
    Ptr->List = "";
}

/*
 * Initialize a PwdHist_t.
 * All strings must be non-NULL for XDR.
 */
void
PwdHistinit(InfoPtr)
    PwdHist_t		       *InfoPtr;
{
    memset(InfoPtr, 0, sizeof(PwdHist_t));
    InfoPtr->UserName = "";
    InfoPtr->List = "";
    ResultInit(&InfoPtr->Result);
}

/*
 * Get name of PWH db file.
 */
char *
PWHgetfile(DomainName, ErrBuff)
    char		       *DomainName;
    char		       *ErrBuff;
{
    static char		        File[MAXPATHLEN];
    static char		        Buff[BUFSIZ];
    char		       *Domain;
    char		       *Err = NULL;

    if (ErrBuff)
	Err = ErrBuff;
    else
	Err = Buff;

    Domain = (DomainName) ? DomainName : GetDomain(NULL);

    if (strlen(PWHdir) + strlen(Domain) + 2 >= sizeof(File)) {
	(void) sprintf(Err, "Size of \"%s\" + \"%.100s\" is too large.", 
		       PWHdir, Domain);
	if (!ErrBuff)
	    Error("%s", Err);
	return((char *)NULL);
    }

    (void) sprintf(File, "%s/%s", PWHdir, Domain);

    return(File);
}

/*
 * Get key from PWH string.
 */
char *
PWHgetkey(String)
    char		       *String;
{
    return(DBgetkeyuser(String));
}

/*
 * Convert a PWHinfo string into an PwdHist_t data type.
 */
PwdHist_t *
PWHgetinfo(String)
    char		       *String;
{
    static PwdHist_t	       *Info = NULL;
    register char      	       *cp;
    register char      	       *Word;

    if (!Info)
	Info = (PwdHist_t *) xmalloc(sizeof(PwdHist_t));
    else {
	if (Info->List && Info->List[0])
	    (void) free(Info->List);
	if (Info->UserName && Info->UserName[0])
	    (void) free(Info->UserName);
    }
    memset((char *)Info, 0, sizeof(PwdHist_t));	/* Zap Info */

    Word = String;
    cp = strchr(Word, ':');
    if (cp)
	*cp++ = CNULL;
    if (Word)
	Info->UserName = strdup(Word);
    else
	Info->UserName = "";			/* For XDR */

    if (cp)
	Info->List = strdup(cp);
    else
	Info->List = "";			/* For XDR */

    return(Info);
}

/*
 * Get pwh entry by matching UserName.
 */
PwdHist_t *
_PWHgetbyuser(DBinfo, UserName)
    DBinfo_t		       *DBinfo;
    char		       *UserName;
{
    static PwdHist_t	        Info;
    static DBarg_t	        DBkey;
    static DBarg_t	        DBresult;
    PwdHist_t		       *PWHptr;
    char		       *StrPtr;
    int				Status;

    errno = 0;
    if (!DBinfo)
	return((PwdHist_t *) NULL);

    DBkey.Data = (void *) UserName;
    DBkey.Size = (size_t) strlen(UserName);
    DBresult.Data = (void *) NULL;
    DBresult.Size = (size_t) 0;

    PwdHistinit(&Info);

    Status = DBget(DBinfo, &DBkey, &DBresult);

    if (Status == 0) {			/* Success */
	StrPtr = (char *) DBresult.Data;
	StrPtr[DBresult.Size] = CNULL;
	if (PWHptr = PWHgetinfo(StrPtr)) {
	    PWHptr->Result.Status = RS_OK;
	    PWHptr->Result.Message = "";/* Required for XDR */
	    return(PWHptr);
	}
    } else if (Status == 1) {		/* No such entry */
	Info.Result.Status = RS_NOENT;
    } else {				/* Error occured */
	Info.Result.Message = SYSERR;
    }

    return(&Info);
}

/*
 * Get pwh entry by matching UserName.
 */
PwdHist_t *
PWHgetbyuser(UserName, Domain)
    char		       *UserName;
    char		       *Domain;
{
    char		       *File;
    static PwdHist_t		Info;
    PwdHist_t		       *InfoPtr;
    DBinfo_t		       *DBinfo;
    static char		        Buff[BUFSIZ];

    /*
     * Prepare error result in case we need it.
     */
    PWHinit(&Info);

    File = PWHgetfile(Domain, Buff);
    if (!File) {
	Info.Result.Message = Buff;
	return(&Info);
    }

    if (!(DBinfo = DBopen(File, O_RDONLY))) {
	if (errno == ENOENT)
	    Info.Result.Status = RS_NOENT;
	(void) sprintf(Buff, "Cannot open %s: %s.", File, SYSERR);
	Info.Result.Message = Buff;
	return(&Info);
    }
	
    InfoPtr = _PWHgetbyuser(DBinfo, UserName);

    (void) DBclose(DBinfo);

    return (InfoPtr);
}

/*
 * Show PWH information about a given username.
 */
int
PWHshow(UserName, Pwh)
    char		       *UserName;
    PwdHist_t		       *Pwh;
{
    if (!Pwh->UserName) {
	Error("No password history information available for \"%s\".", 
	      UserName);
	return(-1);
    }

    printf("Password history information for %s:\n", UserName);
    printf("\tPrevious passwords: %s\n", Pwh->List);

    return(0);
}

/*
 * Read PWH information.
 */
int
PWHread(FileName, DomainName)
    char		       *FileName;
    char		       *DomainName;
{
    char		       *DBfile;
    DBinfo_t		       *DBinfo;
    static DBarg_t	        DBkey;
    static DBarg_t	        DBdata;
    int				Status;
    FILE		       *fp;

    if (!FileName || EQ(FileName, "-"))
	fp = stdout;
    else
	if (!(fp = fopen(FileName, "w+"))) {
	    Error("Cannot open \"%s\" for writing: %s.", FileName, SYSERR);
	    exit(1);
	}

    DBfile = PWHgetfile(DomainName, NULL);
    if (!DBfile) {
	Error("Cannot determine database file name.");
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    if (!(DBinfo = DBopen(DBfile, O_RDONLY))) {
	Error("Cannot open \"%s\" for reading: %s.", DBfile, SYSERR);
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    for (Status = DBiterate(DBinfo, &DBkey, &DBdata, TRUE); Status == 0; 
	 Status = DBiterate(DBinfo, &DBkey, &DBdata, FALSE)) {
	if (DoKeys)
	    (void) fprintf(fp, "%.*s ", DBkey.Size, DBkey.Data);
	(void) fprintf(fp, "%.*s\n", DBdata.Size, DBdata.Data);
    }

    if (Status == -1)
	Error("Database sequential fetch error: %s.", SYSERR);
    else if (Status == 1)	/* Normal "end of entries" indicator */
	Status = 0;

    (void) DBclose(DBinfo);
    if (fp != stdin)
	(void) fclose(fp);

    return(Status);
}

/*
 * Write PWH information
 */
int
PWHwrite(FileName, DomainName)
    char		       *FileName;
    char		       *DomainName;
{
    static char			Buff[MAXDBVAL];
    char		       *DBfile;
    DBinfo_t		       *DBinfo;
    static DBarg_t		DBkey;
    static DBarg_t		DBdata;
    register int		LineNo;
    register int		Len;
    int				Status = -1;
    FILE		       *fp;

    if (!FileName || EQ(FileName, "-"))
	fp = stdin;
    else
	if (!(fp = fopen(FileName, "r"))) {
	    Error("Cannot open \"%s\" for reading: %s.", FileName, SYSERR);
	    exit(1);
	}

    DBfile = PWHgetfile(DomainName, NULL);
    if (!DBfile) {
	Error("Cannot determine database file name.");
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    if (!(DBinfo = DBopen(DBfile, O_RDWR|O_CREAT))) {
	Error("Cannot open \"%s\" for writing: %s.", DBfile, SYSERR);
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    LineNo = 0;
    while (fgets(Buff, sizeof(Buff), fp)) {
	++LineNo;
	Len = strlen(Buff);
	if (Buff[Len - 1] == '\n')
	    Buff[--Len] = CNULL;

	if (!(DBkey.Data = (void *) PWHgetkey(Buff))) {
	    Error("Error: line %d: Cannot parse key.", LineNo);
	    Status = 1;
	    continue;
	}
	DBkey.Size = (size_t) strlen(DBkey.Data);
	DBdata.Data = (void *) Buff;
	DBdata.Size = (size_t) Len;

	if ( DBstore(DBinfo, &DBkey, &DBdata, 0) != 0) {
	    Error("Error: line %d: Database write failed: %s.", 
		  LineNo, SYSERR);
	    Status = 1;
	}
    }

    (void) DBclose(DBinfo);
    if (fp != stdin)
	(void) fclose(fp);

    return(Status);
}

/*
 * Merge Pass into list of entries from PWHptr.
 * The list is colon seperated starting with the most recent entries.
 */
char *
PWHmerge(Pass, UserName, PWHptr)
    char		       *Pass;
    char		       *UserName;
    PwdHist_t		       *PWHptr;
{
    static char		       *Buffer = NULL;
    register char	       *cp;
    register int		Count = 0;

    if (Buffer) {
	(void) free(Buffer);
	Buffer = NULL;
    }

    /*
     * Iterate over the existing list.  If we reach MaxPWHent, then
     * truncate remaining entries.
     */
    if (PWHptr && PWHptr->List && PWHptr->List[0]) {
	for (Count = 1, cp = PWHptr->List; cp && *cp; ++cp)
	    if (*cp == ':')
		if (Count >= MaxPWHent - 1)
		    *cp = CNULL;
		else
		    ++Count;

	Buffer = (char *) xmalloc(strlen(PWHptr->List) + 
				  strlen(UserName) + 
				  strlen(Pass) + 2);
	(void) sprintf(Buffer, "%s:%s:%s", UserName, Pass, PWHptr->List);
    } else {
	Buffer = (char *) xmalloc(strlen(UserName) + 
				  strlen(Pass) + 2);
	(void) sprintf(Buffer, "%s:%s", UserName, Pass);
    }

    return(Buffer);
}

/*
 * Add String to UserName's entry in the Pwd Hist db.
 */
int
PWHadd(UserName, Domain, String)
    char		       *UserName;
    char		       *Domain;
    char		       *String;
{
    static char			ErrBuff[BUFSIZ];
    static DBarg_t	        DBkey;
    static DBarg_t	        DBdata;
    char		       *StrPtr;
    char		       *DBfile;
    int				Status;
    DBinfo_t		       *DBinfo;

    ErrBuff[0] = CNULL;
    DBfile = PWHgetfile(Domain, ErrBuff);
    if (!DBfile) {
	Error("%s", ErrBuff);
	return(-1);
    }

    if (!(DBinfo = DBopen(DBfile, O_RDWR|O_CREAT))) {
	Error("Cannot open \"%s\" for writing: %s.", DBfile, SYSERR);
	return(-1);
    }

    DBkey.Data = (void *) UserName;
    DBkey.Size = (size_t) strlen(UserName);
    DBdata.Data = (void *) String;
    DBdata.Size = (size_t) strlen(String);

    Status = DBstore(DBinfo, &DBkey, &DBdata, 0);
    (void) DBclose(DBinfo);

    if (Status == 0)
	return(0);
    else {
	Error("Write to pwdhist db for user %s failed: %s.", UserName, SYSERR);
	return(-1);
    }
}

/*
 * Insert Pass to UserName's entry in the Pwd Hist db.
 */
int
PWHinsert(UserName, Domain, Pass)
    char		       *UserName;
    char		       *Domain;
    char		       *Pass;
{
    char		       *StrPtr;
    static PwdHist_t		New;
    PwdHist_t		       *PwdHist;

    /*
     * Merge any current entry with the new password.
     */
    PwdHist = PWHgetbyuser(UserName, Domain);
    if (!PwdHist || PwdHist->Result.Status != RS_OK) {
	Error("Get pwdhistory entry for %s@%s failed: %s",
	      UserName, Domain, ResultErrStr(&PwdHist->Result));
	PWHinit(&New);
	New.UserName = UserName;
	PwdHist = &New;
    }
    StrPtr = PWHmerge(Pass, UserName, PwdHist);
    if (!StrPtr) {
	Error("Password history merge of `%s' failed for %s@%s",
	      Pass, UserName, Domain);
	return(-1);
    }

    return(PWHadd(UserName, Domain, StrPtr));
}

/*
 * Set an PWH argument.
 *
 * SetArg is the full (colon seperated) list of encrypted passwords.
 */
int
PWHset(UserName, Domain, SetArg)
    char		       *UserName;
    char		       *Domain;
    char		       *SetArg;
{
    PwdHist_t		       *Info;
    char		       *ErrStr = NULL;

    Info = PWHgetbyuser(UserName, Domain);
    if (Info->Result.Status != RS_OK) {
	if (Debug)
	    Message("Could not get existing pwdhistory entry for %s@%s: %s.",
		    UserName, Domain, ResultErrStr(&Info->Result));
	Info = (PwdHist_t *) xmalloc(sizeof(PwdHist_t));
	PWHinit(Info);
    }
    Info->UserName = UserName;

    Info->List = SetArg;
    if (PWHadd(UserName, Domain, Info->List) < 0) {
	Error("Update of PWH entry for %s@%s failed.", UserName, Domain);
	return(-1);
    }

    /* Success! */
    return(0);
}

/*
 * Delete PWH entry from database.
 */
int
PWHdelete(UserName, Domain)
    char		       *Domain;
    char		       *UserName;
{
    static char			ErrBuff[BUFSIZ];
    static DBarg_t		DBkey;
    char		       *DBfile;
    DBinfo_t		       *DBinfo;
    int				Status = 0;

    ErrBuff[0] = CNULL;
    DBfile = PWHgetfile(Domain, ErrBuff);
    if (!DBfile) {
	Error("%s", ErrBuff);
	return(-1);
    }

    if (!(DBinfo = DBopen(DBfile, O_RDWR|O_CREAT))) {
	Error("Cannot open \"%s\" for writing: %s.", DBfile, SYSERR);
	return(-1);
    }

    DBkey.Data = (void *) UserName;
    DBkey.Size = (size_t) strlen(UserName);

    if (DBdelete(DBinfo, &DBkey) != 0) {
	Error("Database delete failed: %s", SYSERR);
	Status = -1;
    }

    (void) DBclose(DBinfo);

    return(Status);
}
