/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/spm.c,v 1.21 1996/03/02 00:42:35 mcooper Exp $";
#endif	/* lint */

/*
 * Password Management System user program
 */

#include "defs.h"
#include "options.h"

char		       *ConfigFile = SPM_CONF;	/* Configuration file */
char		       *UserName = NULL;	/* Name of user for query */
char		       *ArgStr = NULL;		/* Command line argument */
char		       *DomainName = NULL;	/* Name of domain for query */
int			ChgPwd = 0;		/* Change pwd flag */
int			ChgGecos = 0;		/* Change gecos flag */
int			ChgShell = 0;		/* Change shell flag */
int			DoVersion = 0;		/* Show version info */
int			MaxAsk = MAX_ASK;	/* Max times to ask user */
FACtype_t		FacType = FAC_NIS;	/* Passwd facility type */
char		       *FacArg = NULL;		/* Facility argument */

/*
 * Command line options table.
 */
OptionDescRec Opts[] = {
#if	defined(DEBUG)
    /* These options may be potential security holes */
    {"-conf",	SepArg,		OptStr, 	__ &ConfigFile,		NULL,
	 "<file>",	"Name of configuration file"},
    {"-domain",	SepArg,		OptStr, 	__ &DomainName,		NULL,
	 "<name>",	"Name of domain"},
    {"-server",	SepArg,		OptStr, 	__ &SPMserver,		NULL,
	 "<hostname>",	"Hostname of server"},
#endif	/* DEBUG */
    {"-chpw",	NoArg,		OptBool, 	__ &ChgPwd,		"1",
	 NULL,	"Change password"},
    {"-chfn",	NoArg,		OptBool, 	__ &ChgGecos,		"1",
	 NULL,	"Change full name (GECOS)"},
    {"-chsh",	NoArg,		OptBool, 	__ &ChgShell,		"1",
	 NULL,	"Change login shell"},
    {"-debug",ArgHidden|SepArg,	OptInt, 	__ &Debug,		"1",
	 NULL,	"Enable debugging"},
    {"-facility",SepArg,	OptStr, 	__ &FacArg,		NULL,
	 "<type>",	"Specify which password facility to use"},
    {"-user",	SepArg,		OptStr, 	__ &UserName,		NULL,
	 "<name>",	"Name of user"},
    {"-version",NoArg,		OptBool, 	__ &DoVersion,		"1",
	 NULL,	"Show version information"},
};

/*
 * Run time initialization
 */
static int
Init(Argc, Argv)
    int				Argc;
    char		      **Argv;
{
    register int		i;
    int				Count;
    int				Remain;
    int				Valid;

    ProgramName = Basename(Argv[0], '/');	/* Our program name */
    UseSyslog = 0;				/* Log to syslog()? */

    (void) openlog(ProgramName, LOG_PID, LOG_AUTH);

    CONFinit();
    if (CONFparse(ConfigFile, VarTable, NULL) != 0)
	exit(1);

    Remain = ParseOptions(Opts, Num_Opts(Opts), Argc, Argv);
    if (Remain < 0)
	exit(1);

    if (DoVersion) {
	printf("%s\n", GetVersion());
	exit(0);
    }

    CONFinstall();

    if (FacArg) {
	for (i = 0, Valid = FALSE; !Valid && Facilities[i].Name; ++i)
	    if (strcasecmp(FacArg, Facilities[i].Name) == 0) {
		Valid = TRUE;
		break;
	    }
	if (!Valid) {
	    Error("The name \"%s\" is not a valid facilities type.", FacArg);
	    Error("The valid facility names are:");
	    Error("");
	    Error("\t%-10s %s", "NAME", "DESCRIPTION");
	    for (i = 0; Facilities[i].Name; ++i)
		Error("\t%-10s %s", 
		      Facilities[i].Name, Facilities[i].FullName);
	    exit(1);
	}
	FacType = Facilities[i].Type;
    }

    Count = ChgPwd + ChgGecos + ChgShell;
    if (Count == 0) {
	/* Try to figure out what to do based on our program name */
	if (EQ(ProgramName, "passwd") || EQ(ProgramName, "yppasswd") ||
	    EQ(ProgramName, "nispasswd"))
	    ChgPwd = TRUE;
	else if (EQ(ProgramName, "chsh") || EQ(ProgramName, "ypchsh"))
	    ChgShell = TRUE;
	else if (EQ(ProgramName, "chfn") || EQ(ProgramName, "ypchfn"))
	    ChgGecos = TRUE;
	else {
	    Error(
	  "You must specify one of \"-chpw\", \"-chfn\", or \"-chsh\".");
	    UsageOptions(Opts, Num_Opts(Opts), NULL);
	    return(-1);
	}
    } else if (Count > 1) {
	Error(
      "You can only specify one of \"-chpw\", \"-chfn\", or \"-chsh\".");
	UsageOptions(Opts, Num_Opts(Opts), NULL);
	return(-1);
    }

    /*
     * See if user specified username as an argument.
     */
    if (!UserName && Remain < Argc && Argv[Remain])
	UserName = strdup(Argv[Remain]);

    /*
     * Check for optional argument string
     */
    if (Remain + 1 < Argc)
	ArgStr = strdup(Argv[Remain + 1]);

    return(0);
}

/*
 * Figure out what the user's username is.
 */
char *
GetUserName()
{
    char		       *Name;
    struct passwd	       *Pwd;

    Name = getlogin();
    if (!Name) {
	Pwd = getpwuid(getuid());
	if (!Pwd) {
	    Error("Your uid (%d) is unknown to the system.", getuid());
	    exit(1);
	}
	Name = strdup(Pwd->pw_name);
    }

    return(Name);
}

/*
 * Where all things begin and end.
 */
main(Argc, Argv)
    int				Argc;
    char		      **Argv;
{
    int				Status = 1;
    char		       *ReqUser;

    if (Init(Argc, Argv) < 0)
	exit(1);

    ReqUser = GetUserName();
    if (!UserName)
	UserName = ReqUser;

    if (ChgPwd)
	Status = Change(UserName, ReqUser, FacType, PC_PWD, ArgStr);
    else if (ChgGecos)
	Status = Change(UserName, ReqUser, FacType, PC_GECOS, ArgStr);
    else if (ChgShell)
	Status = Change(UserName, ReqUser, FacType, PC_SHELL, ArgStr);

    exit(Status);
}
