/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/spmadm.c,v 1.14 1996/02/22 21:50:22 mcooper Exp $";
#endif	/* lint */

/*
 * SPM Admin program
 */

#include "defs.h"
#include "options.h"

typedef enum {
    AA_NONE,
    AA_SHOW,
    AA_READ,
    AA_WRITE,
    AA_DELETE,
    AA_SET
} AAtype_t;

char		       *UserName = NULL;	/* Name of user for query */
char		       *ConfigFile = SPMD_CONF;	/* Configuration file */
char		       *DomainName = NULL;	/* Name of domain for query */
char		       *FileName = NULL;	/* Name of file */
char		       *DataType = NULL;	/* Type of data */
char		       *SetArg = NULL;		/* Set info argument */
char		       *DeleteArg = NULL;	/* Delete info argument */
int			DoShow = 0;		/* Show info mode */
int			DoRead = 0;		/* Read info mode */
int			DoWrite = 0;		/* Write info mode */
int			DoVersion = 0;		/* Show version info */
AAtype_t		AAtype = AA_NONE;	/* Adm Action type */

/*
 * Command line options table.
 */
OptionDescRec Opts[] = {
    {"-agedir",	SepArg,		OptStr, 	__ &AgeDir,		NULL,
	 "<name>",	"Name of Age Directory"},
    {"-debug",ArgHidden|SepArg,	OptInt, 	__ &Debug,		"1",
	 NULL,	"Enable debugging"},
    {"-delete",	SepArg,		OptStr, 	__ &DeleteArg,		NULL,
	 "<key>",	"Delete data for <key>"},
    {"-domain",	SepArg,		OptStr, 	__ &DomainName,		NULL,
	 "<name>",	"Name of domain"},
    {"-file",	SepArg,		OptStr, 	__ &FileName,		NULL,
	 "<filename>",	"Name of file to read/write"},
    {"-keys",	NoArg,		OptBool, 	__ &DoKeys,		"1",
	 NULL,		"Show keywords"},
    {"-pddir",	SepArg,		OptStr, 	__ &PDdir,		NULL,
	 "<name>",	"Name of Personal Data Directory"},
    {"-pwdhistdir",SepArg,	OptStr, 	__ &PWHdir,		NULL,
	 "<name>",	"Name of Passwd History Directory"},
    {"-read",	NoArg,		OptBool, 	__ &DoRead,		"1",
	 NULL,		"Read all info from <type> database"},
    {"-show",	NoArg,		OptBool, 	__ &DoShow,		"1",
	 NULL,		"Show information"},
    {"-set",	SepArg,		OptStr, 	__ &SetArg,		NULL,
	 "<key=value>",	"Set information"},
    {"-type",	SepArg,		OptStr, 	__ &DataType,		NULL,
	 "<DataType>",	"Type of data"},
    {"-user",	SepArg,		OptStr, 	__ &UserName,		NULL,
	 "<name>",	"Name of user"},
    {"-version",NoArg,		OptBool, 	__ &DoVersion,		"1",
	 NULL,	"Show version information"},
    {"-write",	NoArg,		OptBool, 	__ &DoWrite,		"1",
	 NULL,		"Write info to <type> database"},
};

static int
Init(Argc, Argv)
    int				Argc;
    char		      **Argv;
{
    int				Remain;
    int				Count = 0;
    char		       *What = NULL;

    ProgramName = Basename(Argv[0], '/');	/* Our program name */
    UseSyslog = 0;				/* Log to syslog()? */

    (void) openlog(ProgramName, LOG_PID, LOG_AUTH);

    if (CONFparse(ConfigFile, VarTable, NULL) != 0)
	exit(1);

    Remain = ParseOptions(Opts, Num_Opts(Opts), Argc, Argv);
    if (Remain < 0)
	exit(1);

    if (DoVersion) {
	printf("%s\n", GetVersion());
	exit(0);
    }

    CONFinstall();

    if (DoShow) {	AAtype = AA_SHOW;	++Count;    	}
    if (DoRead) {	AAtype = AA_READ;	++Count;	}
    if (DoWrite) {	AAtype = AA_WRITE;	++Count;	}
    if (SetArg) {	AAtype = AA_SET;	++Count;	}
    if (DeleteArg) {	AAtype = AA_DELETE;	++Count;	}
    if (Count > 1)
	What = "can only";
    else if (Count == 0)
	What = "must";
    if (What) {
	Error("You %s specify one of: -delete, -read, -write, -show, -set", 
	      What);
	UsageOptions(Opts, Num_Opts(Opts), NULL);
	exit(1);
    }

    if (!DataType) {
	Error("You must specify a data type with the \"-type\" argument.");
	UsageOptions(Opts, Num_Opts(Opts), NULL);
	exit(1);
    }

    /*
     * See if user specified username as an argument.
     */
    if (!UserName && Remain < Argc && Argv[Remain])
	UserName = strdup(Argv[Remain]);

    if ((AAtype == AA_SHOW || AAtype == AA_SET) && !UserName) {
	Error("You must specify a username.");
	UsageOptions(Opts, Num_Opts(Opts), NULL);
	exit(1);
    }

    return(0);
}

main(Argc, Argv)
    int				Argc;
    char		      **Argv;
{
    static char		        Buff[BUFSIZ];
    DBtable_t		       *DBptr;
    int				Status = -1;
    char		       *Domain;

    if (Init(Argc, Argv) < 0)
	exit(1);

    if (!(DBptr = DBgettable(DataType))) {
	Error("No such data type as \"%s\".", DataType);
	DBlist();
	exit(1);
    }

    Domain = CheckDomain(DomainName, Buff);
    if (!Domain) {
	Error("%s", Buff);
	exit(1);
    }

    switch (AAtype) {
    case AA_SHOW:
	Status = (*DBptr->Show)(UserName, Domain);
	break;
    case AA_READ:
	Status = (*DBptr->Read)(FileName, Domain);
	break;
    case AA_WRITE:
	Status = (*DBptr->Write)(FileName, Domain);
	break;
    case AA_SET:
	Status = (*DBptr->Set)(UserName, Domain, SetArg);
	break;
    case AA_DELETE:
	Status = (*DBptr->Delete)(DeleteArg, Domain);
	break;
    }

    exit(Status);
}
