/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/spmproc.c,v 1.40 1996/03/04 18:43:03 mcooper Exp $";
#endif	/* lint */

/*
 * SPM Proceedure calls
 */

#include "defs.h"

#define CLNAME(cl)	((strcmp(cl->HostName, cl->ClientName)) ? \
			 cl->ClientName : NULL)

/*
 * Declarations
 */
static char			Buff[BUFSIZ];
char			       *ErrStr;

static void
LogRequest(HostName, ClientName, AddrStr, User, ReqUser, Domain, Msg)
    char		       *HostName;
    char		       *ClientName;
    char		       *AddrStr;
    char		       *User;
    char		       *ReqUser;
    char		       *Domain;
    char		       *Msg;
{
#define PA(s)	((s) ? s : "")
    Message("%s%s%s [%s] %s@%s%s%s: %s",
	    PA(HostName),
	    (ClientName) ? "/" : "", (ClientName) ? ClientName : "",
	    PA(AddrStr),
	    PA(User), PA(Domain),
	    (ReqUser && *ReqUser) ? " by " : "",
	    (ReqUser && *ReqUser) ? PA(ReqUser) : "",
	    PA(Msg));
#undef PA
}

static void
LogQuery(Query, ClientInfo, Msg)
    SPMquery_t		       *Query;
    ClientInfo_t	       *ClientInfo;
    char		       *Msg;
{
    LogRequest(ClientInfo->HostName, CLNAME(ClientInfo), 
	       ClientInfo->AddrString,
	       Query->UserName, Query->ReqUser, Query->Domain, Msg);
}

/*
 * End of Request processing.
 *
 * The What field should contain two words: First the action taken
 * (check, change) and second is what data the action applies to
 * (pwd, pwdhistory, gecos, shell).
 */
static void
EndRequest(ClientInfo, User, ReqUser, Domain, What, Status, Msg)
    ClientInfo_t	       *ClientInfo;
    char		       *User;
    char		       *ReqUser;
    char		       *Domain;
    char		       *What;
    int				Status;
    char		       *Msg;
{
    static char			Buff[BUFSIZ];

    (void) sprintf(Buff, "%s: %s%s%s",
		   What,
		   (Status == 0) ? "SUCCEEDED" : "FAILED",
		   (Msg && *Msg) ? " - " : "",
		   (Msg && *Msg) ? Msg : "");

    LogRequest(ClientInfo->HostName, CLNAME(ClientInfo), 
	       ClientInfo->AddrString,
	       User, ReqUser, Domain,
	       Buff);

    if (ClientInfo)
	CLNTdestroy(ClientInfo);
}

static void
EndQuery(ClientInfo, Query, What, Status, Msg)
    ClientInfo_t	       *ClientInfo;
    SPMquery_t		       *Query;
    char		       *What;
    int				Status;
    char		       *Msg;
{
    EndRequest(ClientInfo, Query->UserName, Query->ReqUser, Query->Domain, 
	       What, Status, Msg);
}

/*
 * Check a request
 */
char *
CheckRequest(UserName, Domain, CurPass, FacName, FacPtr, ClientInfo)
    char		       *UserName;
    char		       *Domain;
    char		       *CurPass;
    char		       *FacName;
    Facility_t		      **FacPtr;
    ClientInfo_t	       *ClientInfo;
{
    Facility_t		       *Facility;
    int				Status;

    if (!FacName || !FacName[0])
	return("No facility name specified");

    Facility = FAClookup(FacName, TRUE);
    if (!Facility)
	return(Format("Unknown facility name \"%s\"", FacName));
    if (FacPtr)
	*FacPtr = Facility;

    /*
     * Verify password
     */
    Status = PWDverify(UserName, Domain, CurPass, Facility);
    if (Status > 0)
	return("Invalid password");
    else if (Status < 0)
	return("Cannot obtain your password entry");

    return((char *) NULL);
}

/*
 * Process a request
 */
SPMstat_t
ProcRequest(Query, Request, PCtype, ClientInfo, MsgBuff)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
    PCtype_t			PCtype;
    ClientInfo_t	       *ClientInfo;
    char		       *MsgBuff;
{
    char		       *ErrStr;
    Facility_t		       *Facility;
    static Transact_t		Trans;

    ErrStr = CheckRequest(Query->UserName, Query->Domain, Query->CurPass,
			  Query->Facility, &Facility, ClientInfo);
    if (ErrStr) {
	(void) strcpy(MsgBuff, ErrStr);
	return(RS_ERROR);
    }

    /*
     * Call change function
     */
    (void) memset((char *)&Trans, 0, sizeof(Transact_t));
    Trans.UserName = Query->UserName;
    Trans.NewValue = Query->Arg;
    Trans.Domain = Query->Domain;
    Trans.Type = PCtype;

    return FACchange(Facility, &Trans, FALSE, MsgBuff);
}

/*
 * Check for duplicate requests.
 * This can happen due to the short RPC timeout periods used
 * by the client when this server updates large passwd files.
 */
int
IsDupReq(Query, LastResult)
    SPMquery_t		       *Query;
    SPMresult_t		       *LastResult;
{
    static char			LastUser[BUFSIZ];
    static char			LastArg[BUFSIZ];
    int				Status = 1;

    if (LastResult->Status == RS_OK && 
	LastUser[0] && EQ(LastUser, Query->UserName) &&
	LastArg[0] && EQ(LastArg, Query->Arg))
	Status = 1;
    else
	Status = 0;

    /*
     * Save for duplicate checking
     */
    (void) strcpy(LastUser, Query->UserName);
    (void) strcpy(LastArg, Query->Arg);

    return(Status);
}

/*
 * Set password
 */
SPMresult_t *
spmproc_setpwd_1(Query, Request)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
{
    static SPMresult  		Result;
    static char			MsgBuff[BUFSIZ];
    static char			What[] = "change password";
    ClientInfo_t	       *ClientInfo;
    char		       *ErrStr;

    ClientInfo = AUTHcheck(Request, &SPMaccess);
    if (!ClientInfo)
	return((SPMresult_t *) NULL);

    if (IsDupReq(Query, &Result) > 0) {
	LogQuery(Query, ClientInfo, "Ignoring duplicate request");
	return((SPMresult_t *) NULL);
    }
    ResultInit(&Result);

    MsgBuff[0] = CNULL;
    Result.Status = ProcRequest(Query, Request, PC_PWD, ClientInfo, MsgBuff);
    if (Result.Status != RS_OK) {
	EndQuery(ClientInfo, Query, What, -1, Result.Message = MsgBuff);
	return(&Result);
    }

    if (PWHinsert(Query->UserName, Query->Domain, Query->Arg) < 0)
	LogQuery(Query, ClientInfo, 
		 "Could not add password to pwd history db.");

    ErrStr = AGEupdate(Query->UserName, Query->Domain);
    if (ErrStr)
	LogQuery(Query, ClientInfo, 
		 Format("age data not updated: %s", ErrStr));

    /*
     * Success!
     */
    Result.Status = RS_OK;
    if (MsgBuff[0])
	Result.Message = MsgBuff;
    EndQuery(ClientInfo, Query, What, 0, 
/*	     Format("new %s pwd is \"%s\"", Query->Facility, Query->Arg));*/
	     Format("new %s pwd is \"%s\"", Query->Facility, "x"));

    return (&Result);
}

/*
 * Set password - Version 2
 */
SPMresult_t *
spmproc_setpwd_2(Query, Request)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
{
    return(spmproc_setpwd_1(Query, Request));
}

/*
 * Set shell
 */
SPMresult_t *
spmproc_setshell_1(Query, Request)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
{
    static SPMresult  		Result;
    static char			MsgBuff[BUFSIZ];
    static char			What[] = "change shell";
    ClientInfo_t	       *ClientInfo;
    char		       *ErrStr;

    ClientInfo = AUTHcheck(Request, &SPMaccess);
    if (!ClientInfo)
	return((SPMresult_t *) NULL);

    if (IsDupReq(Query, &Result) > 0) {
	LogQuery(Query, ClientInfo, "Ignoring duplicate request");
	return((SPMresult_t *) NULL);
    }
    ResultInit(&Result);

    MsgBuff[0] = CNULL;
    Result.Status = ProcRequest(Query, Request, PC_SHELL, ClientInfo, MsgBuff);
    if (Result.Status != RS_OK) {
	EndQuery(ClientInfo, Query, What, -1, Result.Message = MsgBuff);
	return(&Result);
    }

    /*
     * Success!
     */
    Result.Status = RS_OK;
    if (MsgBuff[0])
	Result.Message = MsgBuff;
    EndQuery(ClientInfo, Query, What, 0, 
	     Format("new %s shell is \"%s\"", Query->Facility, Query->Arg));

    return (&Result);
}

/*
 * Set shell - Version 2
 */
SPMresult_t *
spmproc_setshell_2(Query, Request)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
{
    return(spmproc_setshell_1(Query, Request));
}

/*
 * Set gecos
 */
SPMresult_t *
spmproc_setgecos_1(Query, Request)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
{
    static SPMresult  		Result;
    static char			MsgBuff[BUFSIZ];
    static char			What[] = "change gecos";
    ClientInfo_t	       *ClientInfo;
    char		       *ErrStr;

    ClientInfo = AUTHcheck(Request, &SPMaccess);
    if (!ClientInfo)
	return((SPMresult_t *) NULL);

    if (IsDupReq(Query, &Result) > 0) {
	LogQuery(Query, ClientInfo, "Ignoring duplicate request");
	return((SPMresult_t *) NULL);
    }
    ResultInit(&Result);

    MsgBuff[0] = CNULL;
    Result.Status = ProcRequest(Query, Request, PC_GECOS, ClientInfo, MsgBuff);
    if (Result.Status != RS_OK) {
	EndQuery(ClientInfo, Query, What, -1, Result.Message = MsgBuff);
	return(&Result);
    }

    /*
     * Success!
     */
    Result.Status = RS_OK;
    if (MsgBuff[0])
	Result.Message = MsgBuff;
    EndQuery(ClientInfo, Query, What, 0, 
	     Format("new %s gecos is \"%s\"", Query->Facility, Query->Arg));

    return (&Result);
}

/*
 * Set gecos - Version 2
 */
SPMresult_t *
spmproc_setgecos_2(Query, Request)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
{
    return(spmproc_setgecos_1(Query, Request));
}

/*
 * Check to see if password (Query->Arg) is in the password history list
 * for Query->UserName.
 *
 * Return RS_NOMATCH if the password is not in the history list.
 * Return RS_MATCH if it is in the list.
 */
SPMresult_t *
spmproc_checkpwdhist_1(Query, Request)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
{
    static char		        What[] = "check pwdhistory";
    PwdHist_t		       *PWHptr;
    char		       *Domain;
    char		       *Msg;
    ClientInfo_t	       *ClientInfo;
    static SPMresult  		Result;
    register char	       *Word;
    register char	       *cp;

    /*
     * Prepare result.
     */
    ResultInit(&Result);

    ClientInfo = AUTHcheck(Request, &SPMaccess);
    if (!ClientInfo)
	return((SPMresult_t *) NULL);

    if (!Query || !Query->UserName || !Query->UserName[0] ||
	!Query->UserName || !Query->UserName[0]) {
	EndQuery(ClientInfo, Query, What, -1,
		   Result.Message = "Bad query parameters");
	return(&Result);
    }

    Domain = CheckDomain(Query->Domain, Buff);
    if (!Domain) {
	EndQuery(ClientInfo, Query, What, -1, Result.Message = Buff);
	return(&Result);
    }

    if (ErrStr = CheckRequest(Query->UserName, Query->Domain,
			      Query->CurPass, Query->Facility, NULL,
			      ClientInfo)) {
	EndQuery(ClientInfo, Query, What, -1, Result.Message = ErrStr);
	return(&Result);
    }
	
    PWHptr = PWHgetbyuser(Query->UserName, Domain);
    if (PWHptr->Result.Status == RS_OK) {
	/*
	 * PWHptr->List is a colon seperated list of 
	 * encrypted passwords.
	 */
	Result.Status = RS_NOMATCH;
	for (Word = cp = PWHptr->List; Word && *Word; Word = cp) {
	    cp = strchr(Word, ':');
	    if (cp)
		*cp++ = CNULL;
	    if (EQ(Word, crypt(Query->Arg, Word))) {
		/* This password has been used before */
		Result.Status = RS_MATCH;
		break;
	    }
	}
    } else
	Result.Status = RS_NOENT;

    switch(Result.Status) {
    case RS_NOENT:
	Msg = "No entry for user";
	break;
    case RS_NOMATCH:
	Msg = "Pwd not in history";
	break;
    case RS_MATCH:
	Msg = "Pwd is in history";
	break;
    }

    EndQuery(ClientInfo, Query, What, 0, Msg);

    return (&Result);
}

/*
 * Check Password History - Version 2
 */
SPMresult_t *
spmproc_checkpwdhist_2(Query, Request)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
{
    return(spmproc_checkpwdhist_1(Query, Request));
}

/*
 * Get personel data
 */
PDinfo2_t *
spmproc_getpd(Query, Request)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
{
    static char			What[] = "get personel data";
    static PDinfo2_t		Info;
    PDinfo2_t		       *InfoPtr = NULL;
    PDitem_t		       *Item;
    char		       *Domain;
    ClientInfo_t	       *ClientInfo;

    /*
     * Prepare error result in case we need it.
     */
    PDinfo2Init(&Info);

    ClientInfo = AUTHcheck(Request, &PDaccess);
    if (!ClientInfo)
	return((PDinfo2_t *) NULL);

    if (!Query || !Query->UserName || Query->UserName[0] == CNULL) {
	EndQuery(ClientInfo, Query, What, -1, 
		 Info.Result.Message = "Bad query parameters");
	return(&Info);
    }

    Domain = CheckDomain(Query->Domain, Buff);
    if (!Domain) {
	EndQuery(ClientInfo, Query, What, -1, Info.Result.Message = Buff);
	return(&Info);
    }

    /*
     * Since the PD server may be a different host than the SPM
     * server, it may not have access to password info for this
     * user.
     */
#if	defined(CHECK_PD_REQUEST)
    if (ErrStr = CheckRequest(Query->UserName, Query->Domain,
			      Query->CurPass, Query->Facility, NULL,
			      ClientInfo)) {
	EndQuery(ClientInfo, Query, What, -1, Info.Result.Message = ErrStr);
	return(&Info);
    }
#else	/* !CHECK_PD_REQUEST */
    /*
     * We need to call PWDverify() so that AmPrived can be set.
     */
    (void) PWDverify(Query->UserName, Query->Domain, 
		     Query->CurPass, FAClookup(Query->Facility, TRUE));
#endif	/* CHECK_PD_REQUEST */

    /*
     * Don't return PD info if we're priv'ed so that the priv'ed
     * client doesn't have to enter PD info for verification.
     */
    if (AmPrived) {
	LogQuery(Query, ClientInfo, "privileges GRANTED");
	Info.Result.Status = RS_NOMATCH;
	InfoPtr = &Info;
    } else
	InfoPtr = PDgetbyuser(Query->UserName, Domain);

    if (InfoPtr && InfoPtr->Result.Status == RS_OK) {
	/*
	 * We only want to tell the client what fields are available
	 * for this particuliar entry.  That way the client can 
	 * prompt the user for the right fields.
	 */
#define SET(s)	if (s && *(s)) \
			s = "x"; \
		else \
			s = ""
	 for (Item = InfoPtr->Items; Item; Item = Item->Next)
	     SET(Item->Value);
#undef SET
    }
    if (InfoPtr) {
	/* XXX for XDR */
	InfoPtr->ReqUser = InfoPtr->CurPass = InfoPtr->FacName = "";	
	if (!InfoPtr->Domain)
	    InfoPtr->Domain = Domain;
    }

    EndQuery(ClientInfo, Query, What, 
	     (InfoPtr && InfoPtr->Result.Status == RS_OK) ? 0 : -1, NULL);

    return (InfoPtr);
}

/*
 * Get personel data - Version 1
 */
PDinfo_t *
spmproc_getpd_1(Query, Request)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
{
    PDinfo2_t		       *Info;

    Info = spmproc_getpd(Query, Request);
    if (Info)
	return(PDinfo2to1(Info));
    else
	return((PDinfo_t *) NULL);
}

/*
 * Get personel data - Version 2
 */
PDinfo2_t *
spmproc_getpd_2(Query, Request)
    SPMquery_t 		       *Query;
    struct svc_req 	       *Request;
{
    return(spmproc_getpd(Query, Request));
}

/*
 * Check personel data
 */
SPMresult_t *
spmproc_checkpd(Info, Request)
    PDinfo2_t		       *Info;
    struct svc_req 	       *Request;
{
    char		       *WhatStr;
    static char			What[] = "check personel data";
    static SPMresult_t		Result;
    PDinfo2_t		       *Ptr;
    register PDitem_t	       *CltItem;
    register PDitem_t	       *DBitem;
    char		       *Domain;
    ClientInfo_t	       *ClientInfo;
    register int		ClientCnt;
    register int		MatchCnt;

    ResultInit(&Result);

    ClientInfo = AUTHcheck(Request, &PDaccess);
    if (!ClientInfo)
	return((SPMresult *) NULL);

    if (!Info || !Info->UserName || Info->UserName[0] == CNULL) {
	EndRequest(ClientInfo, Info->UserName, Info->ReqUser,
		   Info->Domain, 
		   What, -1, Result.Message = "Bad query parameters");
	return(&Result);
    }

    Domain = CheckDomain(Info->Domain, Buff);
    if (!Domain) {
	EndRequest(ClientInfo, Info->UserName, Info->ReqUser,
		   Info->Domain, What, -1, Result.Message = Buff);
	return(&Result);
    }

    /*
     * Since the PD server may be a different host than the SPM
     * server, it may not have access to password info for this
     * user.
     */
#if	defined(CHECK_PD_REQUEST)
    if (ErrStr = CheckRequest(Info->UserName, Info->Domain,
			      Info->CurPass, Info->FacName, NULL,
			      ClientInfo)) {
	EndRequest(ClientInfo, Info->UserName, Info->ReqUser, 
		   Info->Domain, What, -1, Result.Message = ErrStr);
	return(&Result);
    }
#endif	/* CHECK_PD_REQUEST */

    Ptr = PDgetbyuser(Info->UserName, Domain);
    MatchCnt = 0;
    ClientCnt = 0;
    if (Ptr && Ptr->Result.Status == RS_OK) {
	/*
	 * First count the number of items we got from the client
	 */
	for (CltItem = Info->Items; CltItem; CltItem = CltItem->Next)
	    if (CltItem->Value && *CltItem->Value)
		++ClientCnt;

	/*
	 * Now check each item from the client for a match with our DB.
	 */
	for (CltItem = Info->Items; CltItem; CltItem = CltItem->Next) {
	    DBitem = PDgetItem(CltItem->Key, Ptr->Items);

	    if (DBitem && CltItem->Value && *(CltItem->Value)) {
		if (EQC(CltItem->Value, DBitem->Value))
		    ++MatchCnt;
		else if (IS_ON(DBitem->Flags, PDF_STRIPSEP) ||
		    IS_ON(CltItem->Flags, PDF_STRIPSEP)) {
		    char	       *cp;
		    char	       *Str1 = NULL;
		    char	       *Str2 = NULL;

		    if (cp = StrStripSep(CltItem->Value, StripSepChars))
			Str1 = strdup(cp);
		    Str2 = StrStripSep(DBitem->Value, StripSepChars);
		    if (Str1 && Str2) {
			/*
			 * Compare various combinations of stripped values
			 */
			if (EQC(Str1, Str2))
			    ++MatchCnt;
			else if (EQC(Str1, DBitem->Value))
			    ++MatchCnt;
			else if (EQC(Str2, CltItem->Value))
			    ++MatchCnt;
		    }
		    if (Str1)
			(void) free(Str1);
		}
	    }

	}
    }

    /*
     * Success is indicated if we found at least one match and the 
     * number of matches equals the number of items the client sent to us.
     */
    if (MatchCnt > 0 && MatchCnt == ClientCnt)
	Result.Status = RS_OK;
    else
	Result.Status = RS_NOMATCH;

    WhatStr = PDwhatSet(Info);
    EndRequest(ClientInfo, Info->UserName, Info->ReqUser, 
	       Info->Domain, What, (Result.Status == RS_OK) ? 0 : -1,
	       Format("verified=%s", (WhatStr) ? WhatStr : "<UNKNOWN>"));

    return (&Result);
}

/*
 * Check personel data - Version 1
 */
SPMresult_t *
spmproc_checkpd_1(PerData, Request)
    PDinfo_t		       *PerData;
    struct svc_req 	       *Request;
{
    PDinfo2_t		       *Info;
    static SPMresult_t	        Result;

    Info = PDinfo1to2(PerData);
    if (Info)
	return(spmproc_checkpd(Info, Request));
    else {
	Result.Status = RS_ERROR;
	Result.Message = "Failure converting PDinfo_t to PDinfo2_t";
	return(&Result);
    }
}

/*
 * Check personel data - Version 2
 */
SPMresult_t *
spmproc_checkpd_2(PerData, Request)
    PDinfo2_t		       *PerData;
    struct svc_req 	       *Request;
{
    return(spmproc_checkpd(PerData, Request));
}

/*
 * Check users password
 */
SPMresult_t *
spmproc_checkpwd_1(Query, Request)
    SPMquery_t		       *Query;
    struct svc_req 	       *Request;
{
    char		       *File;
    static char			What[] = "check pwd";
    static SPMresult_t		Result;
    ClientInfo_t	       *ClientInfo;

    ResultInit(&Result);

    ClientInfo = AUTHcheck(Request, &SPMaccess);
    if (!ClientInfo)
	return((SPMresult *) NULL);

    if (ErrStr = CheckRequest(Query->UserName, Query->Domain,
			      Query->CurPass, Query->Facility, NULL,
			      ClientInfo)) {
	EndQuery(ClientInfo, Query, What, -1, Result.Message = ErrStr);
	return(&Result);
    }

    Result.Status = RS_OK;
    EndQuery(ClientInfo, Query, What, 0, NULL);

    return(&Result);
}

/*
 * Check users password - Version 2
 */
SPMresult_t *
spmproc_checkpwd_2(Query, Request)
    SPMquery_t		       *Query;
    struct svc_req 	       *Request;
{
    return(spmproc_checkpwd_1(Query, Request));
}
