/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/spmtrans.c,v 1.9 1996/03/02 00:44:05 mcooper Exp $";
#endif	/* lint */

/*
 * SPM Transaction processor
 */

#include "defs.h"
#include "options.h"

char		       *ConfigFile = SPMTRANS_CONF;	/* Config file */
int			LogStdout = 0;			/* Log to stdout */
int			DoVersion = 0;		/* Show version info */

/*
 * Command line options table.
 */
OptionDescRec Opts[] = {
    {"-logstdout",	NoArg,	OptBool, 	__ &LogStdout,		"1",
	 NULL,	"Log output to stdout"},
    {"-verbose",	NoArg,	OptBool, 	__ &Verbose,		"0",
	 NULL,	"Do not be verbose in messages"},
    {"+verbose",	NoArg,	OptBool, 	__ &Verbose,		"1",
	 NULL,	"Be verbose in messages"},
    {"-debug",ArgHidden|SepArg,	OptInt, 	__ &Debug,		"1",
	 NULL,	"Enable debugging"},
    {"-transdir",	SepArg,	OptStr, 	__ &TransDir,		NULL,
	 "<name>",	"Name of Transactions Directory"},
    {"-transfaclist",	SepArg,	OptStr, 	__ &TransFacList,	NULL,
	 "<Facility List>","Names of facilities to do transactions on"},
    {"-version",	NoArg,	OptBool, 	__ &DoVersion,		"1",
	 NULL,	"Show version information"},
};

/*
 * Initialize
 */
static int
Init(Argc, Argv)
    int				Argc;
    char		      **Argv;
{
    if (CONFparse(ConfigFile, VarTable, NULL) != 0)
	return(-1);

    if (ParseOptions(Opts, Num_Opts(Opts), Argc, Argv) < 0)
	return(-1);

    if (DoVersion) {
	printf("%s\n", GetVersion());
	exit(0);
    }

    if (!LogStdout) {
	UseSyslog = 1;				/* Log to syslog() */
	ProgramName = Basename(Argv[0], '/');	/* Our program name */
	(void) openlog(ProgramName, LOG_PID, LOG_AUTH);
    }

    CONFinstall();    
    (void) umask(UMASK_MODE);

    return(0);     
}

/*
 * Process all the transactions in TransList.
 */
static int
Process(TransList, Facility)
    Transact_t		       *TransList;
    Facility_t		       *Facility;
{
    static char			ErrBuff[BUFSIZ];
    register Transact_t	       *TransPtr;
    register Transact_t	       *DiscardPtr;
    int				Status = 0;

    for (TransPtr = TransList; TransPtr; TransPtr = TransPtr->Next) {
	ErrBuff[0] = CNULL;
	Status = TRANScheck(TransPtr, Facility, ErrBuff);
	/*
	 * XXX If the TRANScheck did not succeed, we should probably
	 * go through the discard list and try processing those until
	 * success.  For now, we delete them if the first transaction
	 * succeeded.
	 */
	if (Status >= 0) {
	    for (DiscardPtr = TransPtr->Discards; DiscardPtr;
		 DiscardPtr = DiscardPtr->Discards) {
		if (Verbose)
		    Message("%s discard (out of date)", DiscardPtr->Ident);
		(void) TRANSdelete(DiscardPtr);
	    }
	    if (Verbose) {
		if (Status == 0)
		    Message("%s okayed", TransPtr->Ident);
		else
		    Message("%s discarded - %s", TransPtr->Ident, ErrBuff);
	    }
	} else {
	    Error("%s error - %s", TransPtr->Ident, ErrBuff);
	}
	if (Status != 0)
	    /*
	     * Mark the transaction so it's not processed
	     */
	    TransPtr->Flags |= TF_IGNORE;
    }

    /*
     * Perform the changes 
     */
    (void) FACchange(Facility, TransList, TRUE, NULL);

    /*
     * Post change operations
     */
    for (TransPtr = TransList; TransPtr; TransPtr = TransPtr->Next) {
	if (!IS_ON(TransPtr->Flags, TF_IGNORE)) {
	    if (TransPtr->Result.Status == RS_OK) {
		if (Verbose)
		    Message("%s completed", TransPtr->Ident);
	    } else if (TransPtr->Result.Status == RS_ERROR) {
		Error("%s error during processing - %s", 
		      TransPtr->Ident, ResultErrStr(TransPtr->Result));
	    }
	}
	if (TRANSdelete(TransPtr) == 0)
	    if (Verbose) Message("%s deleted", TransPtr->Ident);
    }

    return(Status);
}

/*
 * Scan for transactions
 */
static int
Scan()
{
    static char			Path[MAXPATHLEN];
    static char			ErrBuff[BUFSIZ];
    register Facility_t	       *FacPtr;
    register char	       *DirName;
    DIR			       *DirPtr;
    struct dirent	       *DirEnt;
    size_t			PreLen;
    int				Status = 0;
    static Transact_t	       *TransactList = NULL;
    Transact_t		       *NewTrans;
    char		       *Ident;

    PreLen = strlen(TRANS_PREFIX);

    /*
     * Foreach facility, scan and read each transaction file found in that
     * facility's transaction directory.
     */
    for (FacPtr = Facilities; FacPtr && FacPtr->Name; ++FacPtr) {
	DirName = TRANSgetDir(FacPtr);
	if (!DirName) {
	    if (Debug) Message("Cannot find directory name for \"%s\"",
			       FacPtr->Name);
	    continue;
	}

	DirPtr = opendir(DirName);
	if (!DirPtr) {
	    if (errno != ENOENT)
		Error("Open directory \"%s\" failed: %s", DirName, SYSERR);
	    continue;
	}

	/*
	 * Scan the transaction directory looking for transaction
	 * files (files that start with TRANS_PREFIX.
	 */
	while (DirEnt = readdir(DirPtr)) {
	    if (strncmp(DirEnt->d_name, TRANS_PREFIX, PreLen) != 0) {
		if (Debug) Message("%s/%s skipping", DirName, DirEnt->d_name);
		continue;
	    }

	    if (strlen(DirName) + 1 + strlen(DirEnt->d_name) + 1 >= 
		sizeof(Path)) {
		Error("Scan() Path buffer too small for name \"%s\".",
		      DirEnt->d_name);
		++Status;
		continue;
	    }
	    (void) sprintf(Path, "%s/%s", DirName, DirEnt->d_name);

	    Ident = TRANSgetIdent(Path);
	    if (Verbose) Message("%s file %s", Ident, Path);

	    if (Verbose) Message("%s reading", Ident);
	    ErrBuff[0] = CNULL;
	    NewTrans = TRANSread(Path, FacPtr, ErrBuff);
	    if (NewTrans) {
		if (Verbose) Message("%s adding", Ident);
		TRANSinsert(NewTrans, &TransactList);
	    } else {
		if (ErrBuff[0])
		    Error("%s Error: %s", Ident, ErrBuff);
		++Status;
	    }
	}

	(void) closedir(DirPtr);

	if (Process(TransactList, FacPtr) != 0)
	    ++Status;
    }

    return(Status);
}

/*
 * The start of all things
 */
main(Argc, Argv)
    int				Argc;
    char		      **Argv;
{
    if (Init(Argc, Argv) < 0)
	exit(1);

    if (getuid() != 0) {
	Error("This program must be run as ``root''.");
	exit(1);
    }

    exit(Scan());
}
