/*
 * Copyright 1995, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/spwd.c,v 1.2 1996/02/08 00:44:17 mcooper Exp $";
#endif	/* lint */

/*
 * SPM spwd (Shadow Password) functions
 */

#include "defs.h"

#if	defined(HAVE_SPWD)
/*
 * Convert a line from a shadow password file into a shadow passwd struct.
 */
struct spwd *
SPWDstrtospwd(String)
    char		       *String;
{
    static struct spwd		Shadow;
    register char	       *cp;

    memset(&Shadow, -1, sizeof(Shadow));
    cp = strchr(String, '\n');
    if (cp)
	*cp = CNULL;

    Shadow.sp_namp = strtok(String, ":");
    Shadow.sp_pwdp = strtok(NULL, ":");
    if (cp = strtok(NULL, ":"))	
	Shadow.sp_lstchg = strtol(cp,(char **)NULL,0);
    if (cp = strtok(NULL, ":"))	
	Shadow.sp_min = strtol(cp,(char **)NULL,0);
    if (cp = strtok(NULL, ":"))	
	Shadow.sp_max = strtol(cp,(char **)NULL,0);
    if (cp = strtok(NULL, ":"))	
	Shadow.sp_warn = strtol(cp,(char **)NULL,0);
    if (cp = strtok(NULL, ":"))	
	Shadow.sp_inact = strtol(cp,(char **)NULL,0);
    if (cp = strtok(NULL, ":"))	
	Shadow.sp_expire = strtol(cp,(char **)NULL,0);
    if (cp = strtok(NULL, ":"))	
	Shadow.sp_flag = strtol(cp,(char **)NULL,0);

    return(&Shadow);
}

/*
 * Convert a shadow passwd struct to a string suitable for
 * placement in a shadow passwd file.  We're careful not to
 * set unnecessary fields.
 */
char *
SPWDspwdtostr(Shadow)
    struct spwd 	       *Shadow;
{
    static char			Buff[BUFSIZ];

#define SA(s)	( (s) ? s : "" )			/* String Arg */
#define LA(s)	( (s >= 0) ? ltoa(s) : "" )		/* Long Arg */
#define ULA(s)	( ((long)s != -1) ? ultoa(s) : "" )	/* Unsigned Long Arg */

    if (strlen(SA(Shadow->sp_namp)) + 1 +
	strlen(SA(Shadow->sp_pwdp)) + 1 +
	strlen(LA(Shadow->sp_lstchg)) + 1 +
	strlen(LA(Shadow->sp_min)) + 1 +
	strlen(LA(Shadow->sp_max)) + 1 +
	strlen(LA(Shadow->sp_warn)) + 1 +
	strlen(LA(Shadow->sp_inact)) + 1 +
	strlen(LA(Shadow->sp_expire)) + 1 +
	strlen(ULA(Shadow->sp_flag)) + 1 +
	3 > sizeof(Buff)) {
	Error("Shadow pwd string buffer too small for user \"%s\"", 
	      SA(Shadow->sp_namp));
	return((char *) NULL);
    }

    (void) strcpy(Buff, SA(Shadow->sp_namp));
    (void) strcat(Buff, ":");
    (void) strcat(Buff, SA(Shadow->sp_pwdp));
    (void) strcat(Buff, ":");
    (void) strcat(Buff, LA(Shadow->sp_lstchg));
    (void) strcat(Buff, ":");
    (void) strcat(Buff, LA(Shadow->sp_min));
    (void) strcat(Buff, ":");
    (void) strcat(Buff, LA(Shadow->sp_max));
    (void) strcat(Buff, ":");
    (void) strcat(Buff, LA(Shadow->sp_warn));
    (void) strcat(Buff, ":");
    (void) strcat(Buff, LA(Shadow->sp_inact));
    (void) strcat(Buff, ":");
    (void) strcat(Buff, LA(Shadow->sp_expire));
    (void) strcat(Buff, ":");
    (void) strcat(Buff, ULA(Shadow->sp_flag));
    (void) strcat(Buff, "\n");

#undef SA
#undef LA
#undef ULA

    return(Buff);
}

/*
 * Update spwd string 'String' to have new password 'NewPass'
 */
char *
SPWDupdSpwdStr(String, NewPass)
    char		       *String;
    char		       *NewPass;
{
    struct spwd		       *Shadow;

    Shadow = SPWDstrtospwd(String);
    Shadow->sp_pwdp = NewPass;
    return(SPWDspwdtostr(Shadow));
}

/*
 * Look up UserName in our local shadow password file.
 * We don't use fgetspent() for efficiency reasons.
 */
ShadPass_t *
SPWDgetShadPass(UserName, Facility)
    char		       *UserName;
    Facility_t		       *Facility;
{
    ShadPass_t		       *ShadPassPtr = NULL;
    static ShadPass_t		ShadPass;
    FILE		       *FilePtr;
    register struct spwd       *Shadow = NULL;
    static char			Buff[BUFSIZ];
    register char	       *cp;

    FilePtr = fopen(Facility->ShadowFile, "r");
    if (!FilePtr)
	return(ShadPassPtr);

    while (fgets(Buff, sizeof(Buff), FilePtr)) {
	if (!PWDmatch(UserName, Buff))
	    continue;

	if (cp = strchr(Buff, '\n'))
	    *cp = CNULL;
	Shadow = SPWDstrtospwd(Buff);
	ShadPass.UserName = UserName;
	ShadPass.CrPass = Shadow->sp_pwdp;
	ShadPassPtr = &ShadPass;
	break;
    }

    (void) fclose(FilePtr);

    return(ShadPassPtr);
}

#endif	/* HAVE_SPWD */
