/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/strutil.c,v 1.2 1996/03/01 20:39:14 mcooper Exp $";
#endif	/* lint */

/*
 * String Utility functions
 */

#include "defs.h"

/*
 * Is the character What found in the Sep list?
 */
int
StrIsSep(What, Sep)
    int				What;
    char		       *Sep;
{
    register char	       *cp;

    for (cp = Sep; cp && *cp; ++cp)
	if (*cp == (char) What)
	    return(1);

    return(0);
}

/*
 * Return the next token in String which is seperated by one of the
 * characters found in Sep.  Similiar to strtok(), but we handle back-to-back
 * Sep strings (i.e. an empty field).
 */
char *
StrNext(String, Sep)
    char		       *String;
    char		       *Sep;
{
    static char		       *LastString = NULL;
    char		       *StrPtr;
    char		       *cp;

    if (String)
	StrPtr = String;
    else if (!LastString)
	return((char *) NULL);
    else
	StrPtr = LastString;

    for (cp = StrPtr; cp && *cp; ++cp) {
	if (!StrIsSep(*cp, Sep))
	    continue;
	/* Found the seperator */
	*cp = CNULL;
	LastString = ++cp;
	return(StrPtr);
    }

    /*
     * This is the last time we should be called for
     * this particuliar iteration
     */
    LastString = NULL;

    return(StrPtr);
}

/*
 * Parse a string into an argument vector.
 */
int
StrToArgv(String, Sep, ArgvPtr, Flags)
    char		       *String;
    char		       *Sep;
    char		     ***ArgvPtr;
    int				Flags;
{
    char		      **Argv = NULL;
    register int		Argc = 0;
    char		       *StrPtr;
    char		       *Ptr;
    char		       *EndPtr;
    char		       *End;
    register int		i;

    /*
     * Use a private copy of String so we don't harm it.
     */
    StrPtr = strdup(String);

    /*
     * Count the number of entries we're going to make
     */
    for (Argc = 1, Ptr = StrPtr; Ptr && *Ptr; ++Ptr)
	if (StrIsSep(*Ptr, Sep))
	    ++Argc;

    Argv = (char **) xcalloc((Argc+1), sizeof(char *));

    for (Ptr = StrNext(StrPtr, Sep), i = 0; Ptr && i < Argc;
	 Ptr = StrNext(NULL, Sep), ++i) {
	if (Flags && Flags & SF_RMLEADWS)
	    while (*Ptr == ' ' || *Ptr == '\t')
		++Ptr;

	EndPtr = End = &Ptr[strlen(Ptr)-1];

	if (Flags && Flags & SF_RMTRAILWS)
	    while (*EndPtr == ' ' || *EndPtr == '\t')
		--EndPtr;

	if (EndPtr != End)
	    *(EndPtr+1) = CNULL;

	if (Ptr && *Ptr)
	    Argv[i] = strdup(Ptr);
	else
	    Argv[i] = NULL;

	Argv[i+1] = NULL;
    }

    (void) free(StrPtr);
    if (ArgvPtr)
	*ArgvPtr = Argv;

    return(Argc);
}

/*
 * Destroy and argument vector.
 */
void
DestroyArgv(ArgvPtr, Argc)
    char		     ***ArgvPtr;
    int				Argc;
{
    register int		i;
    char		      **Argv;

    if (!ArgvPtr)
	return;

    for (i = 0, Argv = *ArgvPtr; i < Argc; ++i)
	if (Argv[i])
	    (void) free(Argv[i]);

    if (ArgvPtr && *ArgvPtr) {
	(void) free(*ArgvPtr);
	*ArgvPtr = NULL;
    }
}

/*
 * String all characters found in SepChars from String and return the result.
 * The return value is a pointer to allocated memory that is reused on each
 * call. 
 */
char *
StrStripSep(String, SepChars)
    char		       *String;
    char		       *SepChars;
{
    static char		       *Buffer;
    register char	       *sptr;
    register char	       *bptr;

    if (Buffer)
	(void) free(Buffer);
    bptr = Buffer = (char *) xmalloc(strlen(String));

    for (sptr = String; sptr && *sptr; ++sptr)
	if (!StrIsSep(*sptr, SepChars))
	    *bptr++ = *sptr;
    *bptr = CNULL;

    return(Buffer);
}
