/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/time.c,v 1.6 1996/03/02 00:44:23 mcooper Exp $";
#endif	/* lint */

/*
 * Functions dealing with time.
 */

#include "defs.h"

/*
 * Parse a time String and return the value in as time_t.
 * String can be of format:
 *	+days
 */
time_t
TIMEgetPlus(String, ErrBuff)
    char		       *String;
    char		       *ErrBuff;
    /*ARGSUSED*/
{
    time_t			TimeVal;

    (void) time(&TimeVal);
    if (*String == '+')
	++String;
    			/* #days     * 1min * 1hour * 24hours */
    TimeVal += (time_t) atoi(String) * (60 * 60 * 24);

    return(TimeVal);
}

/*
 * Parse a time String and return the value in as time_t.
 * String can be of format:
 *	mm/dd/yy [hh:mm[:ss]]
 *	mm-dd-yy [hh:mm[:ss]]
 */
time_t
TIMEgetDateTime(String, ErrBuff)
    char		       *String;
    char		       *ErrBuff;
{
    static struct tm		tm;
    register char	       *Str;
    register char	       *cp;
    static char		        Buffer[BUFSIZ];
    time_t			TimeVal;

    (void) memset((char *)&tm, 0, sizeof(struct tm));

    /* Parse the month */
    Str = String;
    for (cp = Str; cp && *cp && isdigit(*cp); ++cp);
    if (*cp != '/' && *cp != '-') {
	if (ErrBuff) (void) sprintf(ErrBuff, "Cannot parse month");
	return((time_t) -1);
    }
    memcpy(Buffer, Str, cp - Str);
    tm.tm_mon = atoi(Buffer) - 1;
    Str = ++cp;

    /* Parse the day of month */
    for (cp = Str; cp && *cp && isdigit(*cp); ++cp);
    if (*cp != '/' && *cp != '-') {
	if (ErrBuff) (void) sprintf(ErrBuff, "Cannot parse day of month");
	return((time_t) -1);
    }
    memcpy(Buffer, Str, cp - Str);
    tm.tm_mday = atoi(Buffer);
    Str = ++cp;

    /* Parse the year */
    for (cp = Str; cp && *cp && isdigit(*cp); ++cp);
    if (*cp != ' ' && *cp != CNULL) {
	if (ErrBuff) (void) sprintf(ErrBuff, "Cannot parse year");
	return((time_t) -1);
    }
    memcpy(Buffer, Str, cp - Str);
    tm.tm_year = atoi(Buffer);

    /* Skip white space seperator */
    for ( ; cp && *cp && isspace(*cp); ++cp);
    if (cp && *cp) {
	/* Time was specified */
	/* Parse hour */
	for (cp = Str; cp && *cp && isdigit(*cp); ++cp);
	if (*cp != ':') {
	    if (ErrBuff) (void) sprintf(ErrBuff, "Cannot parse hour");
	    return((time_t) -1);
	}
	memcpy(Buffer, Str, cp - Str);
	tm.tm_hour = atoi(Buffer);
	Str = ++cp;

	/* Parse minute */
	for (cp = Str; cp && *cp && isdigit(*cp); ++cp);
	if (*cp != ':') {
	    if (ErrBuff) (void) sprintf(ErrBuff, "Cannot parse minute");
	    return((time_t) -1);
	}
	memcpy(Buffer, Str, cp - Str);
	tm.tm_min = atoi(Buffer);
	Str = ++cp;

	/* Parse seconds (if any) */
	if (cp && *cp) {
	    for (cp = Str; cp && *cp && isdigit(*cp); ++cp);
	    if (*cp != CNULL) {
		if (ErrBuff) (void) sprintf(ErrBuff, "Cannot parse seconds");
		return((time_t) -1);
	    }
	    memcpy(Buffer, Str, cp - Str);
	    tm.tm_sec = atoi(Buffer);
	}
    }

    TimeVal = mktime(&tm);
    if (TimeVal == (time_t)-1) {
	if (ErrBuff) 
	    (void) sprintf(ErrBuff, "Cannot convert time string `%s' info.", 
			   String);
	return((time_t) -1);
    } else
	return(TimeVal);
}

/*
 * Parse a time String and return the value in as time_t.
 * String can be of format:
 *	+days
 *	mm/dd/yy [hh:mm[:ss]]
 *	mm-dd-yy [hh:mm[:ss]]
 */
time_t
TIMEparse(String, ErrBuff)
    char		       *String;
    char		       *ErrBuff;
{
    if (*String == '+')
	return(TIMEgetPlus(String, ErrBuff));
    else
	return(TIMEgetDateTime(String, ErrBuff));
}
