/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/userpw.c,v 1.7 1996/03/02 00:45:00 mcooper Exp $";
#endif	/* lint */

/*
 * Facilities for AIX's userpw interface
 */

#include "defs.h"

#if	defined(HAVE_USERPW)
#include <userpw.h>
#include <usersec.h>

/*
 * Get UserName's shadow password entry
 */
ShadPass_t *
USERPWgetShadPass(UserName, Facility)
    char		       *UserName;
    Facility_t		       *Facility;
    /*ARGSUSED*/
{
    ShadPass_t		       *ShadPassPtr = NULL;
    static ShadPass_t		ShadPass;
    struct userpw	       *Userpw;

    if (setpwdb(S_READ) != 0) {
	Error("setuserpw(S_READ) failed: %s", SYSERR);
	return(ShadPassPtr);
    }

    Userpw = getuserpw(UserName);
    if (!Userpw)
	return(ShadPassPtr);

    if (ShadPass.CrPass)
	(void) free(ShadPass.CrPass);
    ShadPass.CrPass = strdup(Userpw->upw_passwd);
    ShadPass.UserName = UserName;
    ShadPassPtr = &ShadPass;

    if (endpwdb() != 0)
	Error("Warning: endpwdb() failed: %s", SYSERR);

    return(ShadPassPtr);
}

/*
 * Change password entries via the userpw API.
 *
 * The changes to perform are listed in TransactList.
 * Only password change types (PCtype) indicated by the Type
 * parameter are processed.
 *
 * Each transaction that is successfully completed, needs to be
 * flagged as such.  This allows the transaction file to be deleted
 * later on, if needed.
 */
SPMstat_t
USERPWchange(TransactList, Type, Facility, MsgBuff)
    Transact_t		       *TransactList;
    PCtype_t			Type;
    Facility_t		       *Facility;
    char		       *MsgBuff;
    /*ARGSUSED*/
{
    static struct userpw        UserpwBuff;
    struct userpw	       *Userpw; 
    register Transact_t	       *TransPtr;
    
    /*
     * See if there's any transactions of type Type in TransactList.
     */
    if (TRANScheckType(TransactList, Type, MsgBuff) <= 0)
	return(RS_OK);

    if (Type == PC_PWD) {
	if (setpwdb(S_READ|S_WRITE) != 0) {
	    (void) sprintf(MsgBuff, "setuserpw(S_READ|S_WRITE) failed: %s", 
			   SYSERR);
	    return(RS_ERROR);
	}

	/*
	 * Process each transaction in TransactList that is enabled.
	 */
	for (TransPtr = TransactList; TransPtr; TransPtr = TransPtr->Next) {
	    if (TRANSskip(TransPtr, Type))
		continue;

	    Userpw = getuserpw(TransPtr->UserName);
	    if (!Userpw) {
		/*
		 * No current entry, so we we create a new one.
		 */
		Userpw = &UserpwBuff;
		(void) strncpy(Userpw->upw_name, TransPtr->UserName, 
			       sizeof(Userpw->upw_name));
		Userpw->upw_flags = 0;
	    }
	    Userpw->upw_passwd = TransPtr->NewValue;
	    Userpw->upw_lastupdate = time((time_t *) 0);

	    if (putuserpw(Userpw) == 0)
		/* Mark it TODO */
		TransPtr->Flags |= TF_TODO;
	    else
		(void) sprintf(MsgBuff, 
			       "putuserpw of `%s' for `%s' failed: %s",
			       TransPtr->NewValue, TransPtr->UserName, SYSERR);
	}

	if (endpwdb() != 0)
	    Error("Warning: endpwdb() failed: %s", SYSERR);

	/*
	 * Mark all the transactions we've done
	 */
	TRANSmark(TransactList, Type);
    } else
	/*
	 * XXX - We should use putuserattr(), but that function 
	 *	 appears to be broken (as of AIX 3.2.5).
	 */
	return(PWDchange(TransactList, Type, Facility, MsgBuff));

    return( (MsgBuff[0]) ? RS_ERROR : RS_OK );
}

#endif	/* HAVE_USERPW */
