/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/verify.c,v 1.16 1996/03/02 00:45:14 mcooper Exp $";
#endif	/* lint */

/*
 * Verification information
 */

#include "defs.h"

static char			Prompt[BUFSIZ];
static char			Answer[BUFSIZ];

/*
 * Start verify session
 */
static ClientInfo_t *
VERIFYstart(Query)
    SPMquery_t		       *Query;
{
    ClientInfo_t	       *Client;

    if (!PDserver) {
	if (Debug) Message("No `PDserver' defined.  Trying `SPMserver' ...");
	if (!(PDserver = GetSPMserver(Query->UserName, Query->Domain))) {
	    Error("No `PDserver' or `SPMserver' defined.");
	    return((ClientInfo_t *) NULL);
	}
    }

    if (!(Client = CLNTconnect(PDserver)))
	return((ClientInfo_t *) NULL);

    return(Client);
}

/*
 * End verify session
 */
static void
VERIFYend(Client)
    ClientInfo_t	       *Client;
{
    CLNTdestroy(Client);
}

/*
 * Check Personel Data with server.
 * Return 0 if data checks out.
 * Return 1 if data is incorrect.
 * Return -1 on error.
 */
static SPMstat
VERIFYcheck(PerData, Client)
    PDinfo2_t		       *PerData;
    ClientInfo_t	       *Client;
{
    static SPMresult_t		Result;
    static char			What[] = "Check personel data";

    if (!PerData->Domain || !PerData->Domain[0])
	PerData->Domain = (DomainName) ? DomainName : GetDomain(NULL);

    if (CLNTcall(Client, SPMPROC_CHECKPD,
		 xdr_PDinfo2, (caddr_t)PerData,
		 xdr_SPMresult, (caddr_t)&Result) < 0) {
	Error("%s call to `%s' failed.", What, Client->HostName);
	return (RS_ERROR);
    }

    return (Result.Status);
}

/*
 * Verify a PDitem_t item with the user.
 */
static VERIFYstat_t
VERIFYitem(Item, PerData, Client, FacName)
    PDitem_t		       *Item;
    PDinfo2_t		       *PerData;
    ClientInfo_t	       *Client;
    char		       *FacName;
{
    register int		i;
    SPMstat			Status;
    PDinfo2_t			Info;
    PDitem_t		       *ItemPtr;
    static PDitem_t		LocItem;

    if (!PerData || !Item || !Item->Key || !Item->Key[0]) {
	if (Debug) Message("No PDinfo2_t information available.");
	return(VS_NOINFO);
    }

    if (Item && Item->Key) {
	/*
	 * Fill in Desc and Example fields from values found in spm.conf.
	 * The server usually does this, but we allow local values to
	 * override.
	 */
	if (ItemPtr = PDgetItem(Item->Key, PDitemList)) {
	    Item->Desc = SV(ItemPtr->Desc);
	    Item->Example = SV(ItemPtr->Example);
	    if (ItemPtr->Flags)
		Item->Flags = ItemPtr->Flags;
	}
    }

    if (Item && (!Item->Desc || !Item->Desc[0])) {
	if (Debug) Message("No Desc information available for `%s'", 
			   Item->Key);
	return(VS_NOINFO);
    }

    PDinfo2Init(&Info);
    Info.UserName = PerData->UserName;
    Info.ReqUser = PerData->ReqUser;
    Info.Domain = PerData->Domain;
    Info.FacName = FacName;

    (void) sprintf(Prompt, "Please enter your %s", Item->Desc);

    for (i = 0; i < AskTries; ++i) {
	if (ASKprompt(Prompt, ": ", Item->Example, NULL, 
		      Answer, sizeof(Answer), Item->Flags) != 0)
	    continue;

	(void) memcpy((char *)&LocItem, (char *)Item, sizeof(LocItem));
	LocItem.Next = NULL;
	LocItem.Value = Answer;
	LocItem.Flags = Item->Flags;
	Info.Items = &LocItem;

	Status = VERIFYcheck(&Info, Client);
	switch (Status) {
	case RS_OK:	/* Success! */
	    return(VS_OKAY);
	case RS_NOENT:	/* This should not normally happen */
	    Error("There is no such information available.");
	    return(VS_NOINFO);
	case RS_ERROR:	/* Error occured */
	    continue;
	default:	/* Incorrect info */
	    Error("That information is incorrect.");
	}
    }

    return(VS_BADINFO);
}

/*
 * Get PD info from server
 */
PDinfo2_t *
VERIFYgetPD(QueryInfo, Client)
    SPMquery_t		       *QueryInfo;
    ClientInfo_t	       *Client;
{
    static SPMquery_t		Query;
    static PDinfo2_t		Info;
    static char			What[] = "Get personel data";

    /*
     * Form SPM query
     */
    QueryInit(&Query);
    Query.UserName = QueryInfo->UserName;
    Query.Facility = QueryInfo->Facility;
    Query.Domain = (QueryInfo->Domain) ? QueryInfo->Domain : GetDomain(NULL);
    /* 
     * Yes, we really need to send CurPass so that
     * the server can perform a check to see if we're Priv'ed
     */
    Query.CurPass = QueryInfo->CurPass;

    (void) memset((char *)&Info, 0, sizeof(PDinfo2_t));

    if (CLNTcall(Client, SPMPROC_GETPD,
		 xdr_SPMquery, (caddr_t)&Query,
		 xdr_PDinfo2, (caddr_t)&Info) < 0) {
	Error("%s call to `%s' failed.", What, Client->HostName);
	return((PDinfo2_t *) NULL);
    }

    if (Info.Result.Status == RS_ERROR) {
	Error("%s query failed: %s", What, ResultErrStr(&Info.Result));
	return((PDinfo2_t *) NULL);
    }

    return(&Info);
}

/*
 * Ask user for verification information.
 * If VerifyList is set, then only check the
 * verification steps listed in VerifyList.
 * Otherwise, check all steps.
 */
VERIFYstat_t
VERIFYuser(Query)
    SPMquery_t		       *Query;
{
    register int		Count;
    register int		i;
    VERIFYstat_t		Status = VS_BADINFO;
    char		       *String = NULL;
    char		       *StringPtr = NULL;
    char		       *VerName;
    int				WhichVer = 0;
    PDitem_t		       *Item;
    PDinfo2_t		       *PerData;
    ClientInfo_t	       *Client;
    static char		      **VerifyNames = NULL;
    static int			VerifyCnt = 0;

    Client = VERIFYstart(Query);
    if (!Client)
	return(VS_ERROR);

    /*
     * We expect VERIFYgetPD() to return NULL on error.
     */
    PerData = VERIFYgetPD(Query, Client);
    if (!PerData) {
	VERIFYend(Client);
	return(VS_ERROR);
    }
    if (PerData->Result.Status != RS_OK) {
	VERIFYend(Client);
	return(VS_NOINFO);
    }
    PerData->ReqUser = Query->ReqUser;

    if (VerifyList) {
	VerifyCnt = StrToArgv(VerifyList, ",", &VerifyNames, SF_RMWS);
	if (!VerifyCnt) {
	    Error("Invalid Verify List: %s.", VerifyList);
	    return(VS_ERROR);
	}
    }

    /*
     * Verify up to VerifyNum items.  If a VerifyList was set (in spm.conf)
     * then verify each item in that list (in order).  Otherwise, verify
     * each item availible in PerData->Items.
     */
    for (Count = 0, i = 0, Item = PerData->Items; Count < VerifyNum; ++i) {
	if (VerifyList) {
	    if (WhichVer >= VerifyCnt)
		/* We're done with list */
		break;
	    VerName = VerifyNames[WhichVer++];
	    Item = PDgetItem(VerName, PerData->Items);
	    if (!Item) {
		Error("No verification info defined for \"%s\".", VerName);
		continue;
	    }
	} else if (!Item)
	    /* There are no PDitems */
	    break;

	Status = VERIFYitem(Item, PerData, Client, Query->Facility);
	/*
	 * Status == VS_OKAY means verify was okay.
	 * Status == VS_NOINFO means no information available.
	 * Status == VS_BADINFO means verify info from user was bad.
	 */
	if (Status == VS_OKAY)
	    ++Count;
	else if (Status == VS_BADINFO)
	    break;

	if (!VerifyList) {
	    Item = Item->Next;
	    if (!Item)
		break;
	}
    }

    if (String)
	(void) free(String);
    if (VerifyNames)
	DestroyArgv(&VerifyNames, VerifyCnt);

    VERIFYend(Client);

    return(Status);
}
